#include "Kangaroo.h"
#include <cmath> //for bending the tail while hopping

Kangaroo::Kangaroo()
{
	animate_smoothly = true;
	waitingForCrouch = false;
	hopping = false;
	flipping = false;
	elevation = 0.0;
	spin = 0.0;
	head_tilt = 0.0;
	hopcounter = 0;
}

void Kangaroo::drawIt()
{
	if( animate_smoothly )
	{
		if(hopping) doHop();
	}
	drawTorso();
}

void Kangaroo::toggleLeftLeg()
{
	if( !hopping )
		left_leg.toggle(animate_smoothly);
}
void Kangaroo::toggleRightLeg()
{
	if( !hopping )
		right_leg.toggle(animate_smoothly);
}

void Kangaroo::toggleHeadSwivel()
{
	head.toggle(animate_smoothly);
}

void Kangaroo::toggleLeftArm()
{
	left_arm.toggle(animate_smoothly);
}

void Kangaroo::toggleTailCurl()
{
	if( !hopping )
		tail.toggle(animate_smoothly);
}

void Kangaroo::toggleTransitionMode()
{
	animate_smoothly = !animate_smoothly;
}

void Kangaroo::toggleHop()
{
	if( !hopping && animate_smoothly )
	{
		if( !left_leg.isRaised() )
		{
			toggleLeftLeg();
			waitingForCrouch = true;
		}
		if( !right_leg.isRaised() )
		{
			toggleRightLeg();
			waitingForCrouch = true;
		}
		if( !tail.atNormalPosition() )
		{
			toggleTailCurl();
			waitingForCrouch = true;
		}
		hopping = true;
	}
}

void Kangaroo::toggleBackflip()
{
	if( !hopping )
	{
		toggleHop();
		flipping = true;
	}
}

void Kangaroo::drawTorso()
{
	const float torso_angle = -50.0;
	//scalings for torso
	const float x = 1.3;
	const float y = 0.6;
	const float z = 0.5;
	glPushMatrix();
		glTranslatef(0.0, elevation, 0.0); // for hopping
		glRotatef(-1*spin, 0.0, 0.0, 1.0); // for the backflip
		glRotatef(torso_angle, 0.0, 0.0, 1.0);
		drawScaleCube(x, y, z); // main body
		glPushMatrix();
			glTranslatef(0.5*x, 0.0, 0.0);
			glRotatef(-1*torso_angle, 0.0, 0.0, 1.0);
			tail.draw(animate_smoothly);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(-0.5*x, 0.0, 0.0); //neck anchor point
			glRotatef(-1*torso_angle, 0.0, 0.0, 1.0);
			glRotatef(-1*head_tilt, 0.0, 0.0, 1.0); //tilts when hopping
			head.draw(animate_smoothly); //and neck, and ears
		glPopMatrix();
		glPushMatrix();
			glTranslatef(-0.5*(x-0.2), -0.5*y, 0.5*z);
			glRotatef(-1*torso_angle, 0.0, 0.0, 1.0); //left arm anchor
			left_arm.draw(animate_smoothly);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(-0.5*(x-0.2), -0.5*y, -0.5*z);
			glRotatef(-1*torso_angle, 0.0, 0.0, 1.0); //right arm anchor
			right_arm.draw(animate_smoothly);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.3*x, -0.3*y, 0.5*z); 
			glRotatef(-1*torso_angle, 0.0, 0.0, 1.0);
			left_leg.draw(animate_smoothly);// left leg
			glTranslatef(0.0,0.0,-1*z);
			right_leg.draw(animate_smoothly); // right leg
		glPopMatrix();
	glPopMatrix();
}

void Kangaroo::doHop()
{
	if( waitingForCrouch )
	{
		if( left_leg.isRaised() && right_leg.isRaised() && tail.atNormalPosition() )
			waitingForCrouch = false;
		else
		{
			if(!left_leg.isRaised()) left_leg.toggle(animate_smoothly);
			if(!right_leg.isRaised()) right_leg.toggle(animate_smoothly);
			if(!tail.atNormalPosition()) tail.toggle(animate_smoothly);
			return;
		}
	}
	const float crouch_dist = 0.2052 + 0.5655; //calculated from kleg.cpp
	const float crouch_delta = crouch_dist / 15 / 2; //leg animation is 15 frames
	
	// push the ear back with a parabolic function
	float x = hopcounter - 30.0;
	float y = ((-1.0/60.0)*x*x + 15.0)/15.0;
	head.pushEarBack(y);
	if( flipping ) //use same parabola for backflipping
	{
		if( hopcounter < 30 )
			spin = y*180;
		else if( hopcounter < 60 )
			spin = (2 - y)*180;
	}
	
	// bend the tail with a sine wave
	x = hopcounter * M_PI / 30.0;
	y = std::sin(x+M_PI);
	tail.setAngle(y);
	// tilt the head with the same wave
	head_tilt = y * 15;
	
	if( hopcounter == 0 )
	{
		left_leg.toggle(true);
		right_leg.toggle(true);
	}
	else if( hopcounter < 15 )
	{
		elevation += crouch_delta;
	}
	else if( hopcounter < 45 )
	{
		int y = 30 - hopcounter;
		float jump_delta = (y / 15.0)*crouch_delta;
		if( !flipping ) elevation += jump_delta;
		else elevation += 2*jump_delta;
	}
	else if( hopcounter == 45 )
	{
		left_leg.toggle(true);
		right_leg.toggle(true);
	}
	else if( hopcounter < 60 )
	{
		elevation -= crouch_delta;
	}
	else
	{
		hopcounter = 0;
		hopping = false;
		flipping = false;
		elevation = 0; // correct the position
		spin = 0;
		head_tilt = 0;
		tail.setAngle(0);
		return;
	}
	hopcounter++;
}
