#include "blobtex.h"

BlobTexture* BlobTexture::singleton = NULL;
BlobTexture* BlobTexture::getBlobTexture()
{
	static bool instantiated = false;
	if( instantiated )
	{
		return singleton;
	}
	else
	{
		singleton = new BlobTexture();
		instantiated = true;
		return singleton;
	}
}

BlobTexture::BlobTexture()
{
	const float middle = 15.5;
	for(int x = 0; x < 32; x++)
	for(int y = 0; y < 32; y++)
	{
		float dx = (float)x - middle;
		float dy = (float)y - middle;
		float sumsq = dx*dx + dy*dy;
		float dist = sqrt( sumsq );
		if( dist > middle )
		{
			data[x][y] = 0;
		}
		else
		{
			float alpha = dist / middle;
			alpha = 1 - alpha;
			data[x][y] = char(alpha * 255.0f);
		}
	}
}

void BlobTexture::bind()
{
	static bool built = false;
	if( built )
	{
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glBindTexture(GL_TEXTURE_2D, tex);
	}
	else
	{
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glGenTextures(1, &tex);
		glBindTexture(GL_TEXTURE_2D, tex);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_ALPHA, 32, 32, GL_ALPHA, GL_UNSIGNED_BYTE, data);
		built = true;
	}
}
