#include "clouds.h"

Clouds::~Clouds()
{
	gluDeleteQuadric(sphere);
}

void Clouds::init()
{
	PerlinNoise2D noise2(256, 128);
	PerlinNoise2D noise16(256, 16);
	PerlinNoise2D noise32(256, 8);
	PerlinNoise2D noise64(256, 4);
	float noise[256][256];
	for(int x = 0; x < 256; x++)
	for(int y = 0; y < 256; y++)
	{
		float a = noise16[x][y];
		float b = noise32[x][y];
		float c = noise64[x][y];
		float d = noise2[x][y];
		noise[x][y] = c + b/2 + a/4 + d/16;
	}
	//this exponential filter comes from
	//http://www.gamedev.net/reference/articles/article2085.asp
	const float cover = 125.0;
	const float sharpness = 0.95;
	for(int x = 0; x < 256; x++)
	for(int y = 0; y < 256; y++)
	{
		float z = (noise[x][y] + 1.0) / 2.0;
		z *= 255.0;
		float c = z - cover;
		if(c < 0) c = 0;
		z = 255.0 - (pow(sharpness,c)*255.0);
		clouds[x][y][3] = (char)z;
		clouds[x][y][0] = 117;
		clouds[x][y][1] = 185;
		clouds[x][y][2] = 255;
	}
	
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &texname);
	glBindTexture(GL_TEXTURE_2D, texname);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, 256, 256, GL_RGBA, GL_UNSIGNED_BYTE, clouds);
	
	sphere = gluNewQuadric();
	gluQuadricNormals(sphere, GLU_NONE);
	gluQuadricTexture(sphere, GL_TRUE);
	
	dlist = glGenLists(1);
	glNewList(dlist, GL_COMPILE);
		glDisable(GL_LIGHTING);
		glEnable(GL_TEXTURE_2D);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		GLfloat colors[] = {1,1,1,0};
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
		glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, colors);
		glBindTexture(GL_TEXTURE_2D, texname);
			
		glColor3f(1,1,1);
		gluSphere(sphere, RIGHT, 100, 100);
		
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
		
		//so as not to conflict with alex
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glEndList();
	
	step = 0.0;
	timer = MyTime(1/10.0);
}

void Clouds::draw()
{
	glPushMatrix();
	glTranslatef(RIGHT / 2.0, 0, FAR / 2.0);	
	
	glRotatef(step, 0, 0, 1);
	glCallList(dlist);
	if(timer.check())
	{
		step+=0.25;
		if(step >= 360.0) step = 0.0;
	}
	glPopMatrix();
}



