#include "cockpit.h"
#include "common.h"
extern TEXTURE metalTex;
extern TEXTURE panelTex;
extern TEXTURE dialTex;

Cockpit::Cockpit()
{
	thickness = 3;
	Corner v1;
	v1.set(-12,0,0);
	Corner v2;
	v2.set(12,0,0);
	Corner v3;
	v3.set(7,10,0);
	Corner v4;
	v4.set(-7,10,0);
	Corner v5;
	v5.set(-12,0,thickness * -1);
	Corner v6;
	v6.set(12,0,thickness * -1);
	Corner v7;
	v7.set(7,10,thickness * -1);
	Corner v8;
	v8.set(-7,10,thickness * -1);
	panel.set(&v1,&v2,&v3,&v4,&v5,&v6,&v7,&v8);
	enemyInTarget = false;
}

Cockpit::~Cockpit()
{
	gluDeleteQuadric(panelquad);
}

void Cockpit::init(Object* hero)
{
	panelquad = gluNewQuadric();
	gluQuadricNormals(panelquad, GLU_SMOOTH);
	gluQuadricDrawStyle(panelquad, GLU_FILL);
	diskquad = gluNewQuadric();
	gluQuadricNormals(diskquad, GLU_SMOOTH);
	gluQuadricDrawStyle(diskquad, GLU_FILL);
	gluQuadricTexture(diskquad, GL_TRUE);
	
	this->hero = hero;
}
void Cockpit::drawPanel()
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, metalTex.TexID);
	panel.draw();
	//glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	if(enemyInTarget)
		glColor3f(1,0.0,0.0);
	else
 		glColor3f(1,0.8,0.8);
	
	glTranslatef(0,11.5,0);
	DrawTexturedBox(2.5,0.8,0.6);
	glPopMatrix();
	
	//glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, dialTex.TexID);
	
	glPushMatrix();
		glTranslatef(-4.5, 5.5, 0);
		glColor3f(0.9,0.9, 0.9);
		glDisable(GL_TEXTURE_2D);
		gluCylinder(panelquad, 3, 3, 0.5, 20, 2);
		glTranslatef(0,0,0.5);
		glColor3f(1, 1, 1);
		glEnable(GL_TEXTURE_2D);
		gluDisk(diskquad, 0, 3, 20, 20);
		//health goes from 0 to 100
		glColor3f(0,0,1);
		float hp_rot = hero->hitPoints / 100.0;
		glTranslatef(0,0,0.1);
		glRotatef(135, 0,0,1);
		glRotatef(hp_rot * -270, 0,0,1);
		glDisable(GL_TEXTURE_2D);
		glBegin(GL_LINES);
			glVertex3f(0,0,0);
			glVertex3f(0,2.7,0);
		glEnd();
		glEnable(GL_TEXTURE_2D);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(4.5, 5.5, 0);
		glColor3f(0.9,0.9, 0.9);
		glDisable(GL_TEXTURE_2D);
		gluCylinder(panelquad, 3, 3, 0.5, 20, 2);
		glTranslatef(0,0,0.5);
		glColor3f(1, 1, 1);
		glEnable(GL_TEXTURE_2D);
		gluDisk(diskquad, 0, 3, 20, 20);
		//speed goes from -100 to 100
		glColor3f(1,0,0);
		glTranslatef(0,0,0.1);
		glRotatef(135,0,0,1);
		static float last_speed = 0.0;
		float new_speed = hero->speed;
		if( new_speed - last_speed < 5.0 && new_speed - last_speed > -5.0 )
			new_speed = last_speed;
		else
			last_speed = new_speed;
		float speed_rot = (new_speed+hero->maxSpeed) / (hero->maxSpeed*2);
		glRotatef(speed_rot * -270, 0, 0, 1);
		glDisable(GL_TEXTURE_2D);
		glBegin(GL_LINES);
			glVertex3f(0,0,0);
			glVertex3f(0,2.7,0);
		glEnd();
	glPopMatrix();
	
	//glDisable(GL_TEXTURE_2D);
}	

void Cockpit::drawArms()
{
	glColor3f(0.6,0.6,0.6);	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, metalTex.TexID);
	glPushMatrix();
	glTranslatef(-7,10,-thickness/2);
	glRotatef(15,0,0,1);
	glRotatef(15,1,0,0);
	glTranslatef(0,12,0);
	
	DrawTexturedBox(1.5,40,0.8);
	if(enemyInTarget)
		glColor3f(1,0.0,0.0);
	else
 		glColor3f(1,0.8,0.8);
	glTranslatef(0,0,0.7);
	DrawTexturedBox(1.0,4,0.5);
	
	glPopMatrix();
	
	//draw right arm
	glColor3f(0.6,0.6,0.6);	
	glPushMatrix();
	glTranslatef(7,10,-thickness/2);
	glRotatef(-15,0,0,1);
	glRotatef(15,1,0,0);
	glTranslatef(0,12,0);
	DrawTexturedBox(1.5,40,0.8);
	glTranslatef(0,0,0.7);
	if(enemyInTarget)
		glColor3f(1,0.0,0.0);
	else
 		glColor3f(1,0.8,0.8);
	
	DrawTexturedBox(1.0,4,0.5);
	
	glPopMatrix();
	glDisable(GL_TEXTURE_2D);	
	glColor3f(0.6,0.6,0.6);	
}
void Cockpit::draw()
{
	glPushMatrix();	
	glScalef(0.1,0.1,0.1);
	glTranslatef(0,-22,0);
	glPushMatrix();
	glRotatef(20,1,0,0);
	//drawNormals();
	glColor3f(0.95,0.95,0.95);	
	drawPanel();
	glColor3f(0.6,0.6,0.6);	
	//draw left arm
	drawArms();
	glPopMatrix();	
	glPopMatrix();
	
}
