#include "collisionmanager.h"
#include "hero.cpp"
#include "eviltank.cpp"

CollisionManager::CollisionManager()
{
}

CollisionManager::CollisionManager(Terrain * terrain)
{
	this->terrain = terrain;
}

CollisionManager::~CollisionManager(void)
{

}
void CollisionManager::RunCollisionsCheck(Object * object)
{
	object->collisionInfo.collisionFound = false;
	int numObjects = (int)this->collisionList.size();
	double maxTestDistance=  50;
	int pathSlices = (int)ceil(object->speed/maxTestDistance);
	pathSlices = max(pathSlices,3);
	Vertex * sourcePt = & object->position;
	Vertex velocity = object->unitVelocity.mul(object->speed);
	Vertex finalDest = sourcePt->add(&velocity);
	Arrow pathIncrement = velocity.mul(1.0/pathSlices);
	Arrow radiusArrow;
	Vertex actualTestPt;
	Vertex prevPoint =* sourcePt;
	Vertex nextPoint =  *sourcePt;
	//check for colllisions with objects
	Object * currentObj;
	for(int j =0;j< pathSlices;j++)
	{
		for(int i =0;i < numObjects;i++)
		{
			currentObj = collisionList[i];
			if(currentObj == object || currentObj == object->parent || currentObj->parent == object
				|| !currentObj->collisionInfo.checkAgainstMe
				)
				continue;
	
			double allowedDistance = currentObj->collisionInfo.boundingRadius + object->collisionInfo.boundingRadius;
			Arrow diff = nextPoint.sub(&currentObj->position);		
			double actualDistance = diff.getLength();
			if(actualDistance < allowedDistance)
			{
				object->collisionInfo.collisionFound = true;	
				diff.setLength(1);
				Arrow pathAdjust = diff.sub(&object->unitVelocity);
				pathAdjust.setLength(0.01);
	
				object->collisionInfo.slideDirection = object->unitVelocity.add(&pathAdjust);
				object->collisionInfo.slideDirection.setLength(1);
				diff = diff.mul(allowedDistance * 1.05);
				object->collisionInfo.lastGoodPoint = currentObj->position.add(&diff);
				currentObj->hit(object);
				return;
			}
		}
		nextPoint = nextPoint.add(&pathIncrement);
	}
	if(object->isFlying)
	{
		//check for collisions with ground
		double groundY;
		double pathY;
		Arrow groundNormal;
		prevPoint = *sourcePt;
		nextPoint = prevPoint.add(&pathIncrement);
		actualTestPt = nextPoint;

		for(int i =0; i < pathSlices;i++)
		{
			terrain->getHeightAndNormal(actualTestPt.getX(),actualTestPt.getZ(),&groundY,&groundNormal);
			radiusArrow = groundNormal.mul(-1);
			radiusArrow.setLength(1);
			radiusArrow = radiusArrow.mul(object->collisionInfo.boundingRadius);
   			actualTestPt = nextPoint.add(&radiusArrow);
			//do this again now getting y value at the point where bounding sphere intersects
			terrain->getHeightAndNormal(actualTestPt.getX(),actualTestPt.getZ(),&groundY,&groundNormal);
			pathY = actualTestPt.getY();

			if(groundY >= pathY)
			{
				Arrow sideTgt = groundNormal.cross(object->unitVelocity);
				Arrow pathAdjust = sideTgt.cross(groundNormal);
				pathAdjust.setLength(1);
				pathAdjust = pathAdjust.sub(&object->unitVelocity);
				pathAdjust.setLength(0.001);
				object->collisionInfo.slideDirection = object->unitVelocity.add(&pathAdjust);
				
				object->collisionInfo.slideDirection = object->unitVelocity;
				object->collisionInfo.slideDirection.setLength(1);

				object->collisionInfo.lastGoodPoint = prevPoint;			
				object->collisionInfo.lastGoodPoint.setY(groundY + object->collisionInfo.boundingRadius * 1.15);
				object->collisionInfo.collisionFound = true;
				return;
			}
			prevPoint = nextPoint;
			nextPoint = nextPoint.add(&pathIncrement);
		}	
	}
	//check water bound
	Arrow t(0,object->collisionInfo.boundingRadius,0);
	actualTestPt = finalDest.sub(&t);
	if(actualTestPt.getY() <= 1501)
	{
		
		Arrow sideTgt = object->yAxis.cross(object->unitVelocity);
		Arrow pathAdjust = sideTgt.cross(object->yAxis);
		pathAdjust.setLength(1);
		pathAdjust = pathAdjust.sub(&object->unitVelocity);
		pathAdjust.setLength(0.01);
		object->collisionInfo.slideDirection = object->unitVelocity.add(&pathAdjust);
		t.setLength(0.1);
		object->collisionInfo.lastGoodPoint = sourcePt->add(&t);;				
		object->collisionInfo.collisionFound = true;	
		return;
	}
	
	
		
}
bool CollisionManager::rayToSphereIntersect(Vertex * start, Arrow * dir, Vertex * sphereCenter, double radius,double * distance) 
{
	Arrow t;
	double tempResult;
	dir->setLength(1);
	t.setX(sphereCenter->getX() - start->getX());
	t.setY(sphereCenter->getY() - start->getY());
	t.setZ(sphereCenter->getZ() - start->getZ());
	Arrow q = t;
		
	double c = q.getLength();
	double v = q.dot(dir);
	double d = radius*radius - (c*c - v*v);

	// If there was no intersection, return -1
	if (d < 0.0) return (false);

	// Return the distance to the [first] intersecting point
	tempResult= v - sqrt(d);
	if(tempResult < 0)
		return false;
	*distance = tempResult;
	return true;
}

bool CollisionManager::findObjectHit(Vertex start, Arrow dir,Vertex * resPoint,Object * *resObj,Object * askingObj)

{
	int numObjects = (int)this->collisionList.size();
	double objDistance ;
	double terrainDistance ;
	bool objIntersectionFound = false;
	bool terrainIntersectionFound = false;

	for(int i =0;i < numObjects;i++)
	{
		Object * currObject = this->collisionList[i];
		if(currObject == askingObj || currObject == askingObj->parent || currObject->parent == askingObj
			|| !currObject->collisionInfo.checkAgainstMe
			)
			continue;
		if(rayToSphereIntersect(&start,&dir,&currObject->position,currObject->collisionInfo.boundingRadius,&objDistance))
		{
			Vertex t = dir.mul(objDistance);
			t = t.add(&start);
			* resPoint = t;
			* resObj = currObject;
			return true;	
		}	
	}
	return false;

}




