#ifndef CORNER_CPP
#define CORNER_CPP
#include "common.h"
#include "geometry.cpp"
#include "face.cpp"

class Corner : public Vertex
/*
	This class represents a corner in my terrain. After all faces are generated it has a list 
	of pointers to each face thats adjacent to it
*/
{
public:
	Arrow normal;
	std::vector<Face* > connectedFaces;
	Color cornerColor;

	Corner()
	{		
	}
	
	Corner(double x, double y, double z) : Vertex(x,y,z)
	{
	}
	~Corner()
	{
	
	}
	void addFace(Face * f)
	{
		connectedFaces.push_back(f);
	}
	void clearFaces()
	{
		connectedFaces.clear();
	}
	void calculateNormal() //calculates normal based on adjacent faces
	{
		int numFaces = (int)connectedFaces.size();
		double x=0;
		double y=0;
		double z=0;
		for(int i=0;i < numFaces;i++)
		{
			x += connectedFaces[i]->normal.getX();
			y += connectedFaces[i]->normal.getY();
			z += connectedFaces[i]->normal.getZ();
		}
		x /= numFaces;
		y /= numFaces;
		z /= numFaces;
		normal.set(x,y,z);
		normal.setLength(1);
	}
	void calculateColor() //calculates color based on adjacent faces colors
	{
		double maxHeight = 19000;
		double yCoord = getY();	
		double fract = yCoord /maxHeight;
		fract = fract * fract;
		Vertex startColor(137/255.0,167/255.0,48/255.0);
		Vertex endColor(199/255.0,154/255.0,20/255.0);
		Arrow diff = endColor.sub(&startColor);

		cornerColor.set(startColor.getX() + diff.getX() * fract,
						startColor.getY() + diff.getY() * fract,
						startColor.getZ() + diff.getZ() * fract				
						);


	}
	Arrow * getNormal()
	{
		return &normal;
	}
};
#endif



