#include "common.h"
#include "geometry.cpp"
#include "object.cpp"
#include "custombox.cpp"
#include "terrain.cpp"
#include "texture.h"
#include "hero.cpp"
#include "collisionmanager.h"
#include "perlin2d.h"
#include "smoke.h"
#include "fire.h"
#include "missilemanager.h"
#include "mytime.h"

extern TEXTURE camoTex;
extern TEXTURE metalTex;
extern TEXTURE wheelTex;

class EvilChopper : public Object
{
public:
	Hero * pHero;
	CollisionManager * pCollisionManager;	
	GLUquadric* quad;
	CustomBox bottom, mid, top, window, arm, gunarm, tailBox;
	GLuint texname;
	unsigned char colors[256][256][3];
	float rot_angle;
	Terrain * terrain;
	Smoke smoke;
	Fire fire;
	MyTime timer;
	MissileManager* mmanager;
	EvilChopper(void)
	{
		isFlying = true;
		this->collisionInfo.boundingRadius = 480;
		maxSpeed = 60;
		isEnemy = true;
		timer = MyTime(2);
	}
	void init(Hero * hero,CollisionManager * collisionManager,Vertex * startPos,Terrain * terrain,MissileManager * mmanager)
	{
		this->pHero = hero;		
		this->mmanager = mmanager;
		rot_angle = 0;
		this->terrain = terrain;
		quad = gluNewQuadric();
		gluQuadricNormals(quad, GLU_SMOOTH);
		gluQuadricDrawStyle(quad, GLU_FILL);
		gluQuadricTexture(quad, GL_TRUE);
		
// 		PerlinNoise2D noise2(256, 128);
// 		PerlinNoise2D noise16(256, 16);
// 		PerlinNoise2D noise32(256, 8);
// 		PerlinNoise2D noise64(256, 4);
// 		float noise[256][256];
		PerlinNoise2D noise(256, 4);
// 		for(int x = 0; x < 256; x++)
// 		for(int y = 0; y < 256; y++)
// 		{
// 			float a = noise16[x][y];
// 			float b = noise32[x][y];
// 			float c = noise64[x][y];
// 			float d = noise2[x][y];
// 			noise[x][y] = c + b/2 + a/4 + d/16;
// 		}
		const float cover = 125.0;
		const float sharpness = 0.95;
		for(int x = 0; x < 256; x++)
		for(int y = 0; y < 256; y++)
		{
			float z = (noise[x][y] + 1.0) / 2.0;
			z *= 255.0;
			float c = z - cover;
			if(c < 0) c = 0;
			z = 255.0 - (pow(sharpness,c)*255.0);
			colors[x][y][0] = 200;
			colors[x][y][1] = (char)z;
			colors[x][y][2] = 0;
		}
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glGenTextures(1, &texname);
		glBindTexture(GL_TEXTURE_2D, texname);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, 256, 256, GL_RGB, GL_UNSIGNED_BYTE, colors);
		
		pCollisionManager = collisionManager;
		
		Corner d(-0.5,-0.5,0);
		Corner dn(0.5,-0.5,0);
		Corner h(-0.5, -1, -2.5);
		Corner hn(0.5, -1, -2.5);
		Corner e(-0.5, -1, -1.3);
		Corner en(0.5, -1, -1.3);
		Corner k(-0.5, -0.5, -3.5);
		Corner kn(0.5, -0.5, -3.5);
		bottom.set(&h, &hn, &en, &e, &k, &kn, &dn, &d);
			
		Corner a(-0.5,0.5,0);
		Corner an(0.5, 0.5, 0);
		Corner j(-0.5, 0.5, -3.5);
		Corner jn(0.5, 0.5, -3.5);
		mid.set(&d, &dn, &an, &a, &k, &kn, &jn, &j);
		
		Corner g(-0.5, -0.5, -1);
		Corner gn(0.5, -0.5, -1);
		top.set(&h, &hn, &en, &e, &k, &kn, &gn, &g);
		
		window.set(&d, &dn, &dn, &d, &g, &gn, &en, &e);
		
		a.set(0,0,0);
		Corner ab(0,-0.1,0);
		
		Corner b(0,0,0.5);
		Corner bb(0,-0.1,0.5);
		
		Corner c(-1,0,0.3);
		Corner cb(-1, -0.1, 0.3);
		
		d.set(-1, 0, 0);
		Corner db(-1, -0.1, 0);
		
		arm.set(&cb, &bb, &ab, &db, &c, &b, &a, &d);
		
		e.set(-0.6, -0.1, 0.4);
		Corner eb(-0.6, -0.1, 0.1);
		
		Corner f(-0.6, -0.3, 0.4);
		Corner fb(-0.6, -0.3, 0.1);
		
		g.set(-0.5, -0.3, 0.4);
		Corner gb(-0.5, -0.3, 0.1);
		
		h.set(-0.5, -0.1, 0.4);
		Corner hb(-0.5, -0.1, 0.1);
		
		gunarm.set(&e, &f, &g, &h, &eb, &fb, &gb, &hb);
		
		a.set(0.5, -0.5, 0.0);
		 ab.set(-0.5, -0.5, 0.0);
		 b.set(0.2, -0.2, -3.0);
		 bb.set(-0.2, -0.2, -3.0);
		 c.set(0.2, 0.2, -3.0);
		 cb.set(-0.2, 0.2, -3.0);
		 d.set(0.5, 0.5, 0.0);
		 db.set(-0.5, 0.5, 0.0);
		
		tailBox.set(&a, &b, &c, &d, &ab, &bb, &cb, &db);
		
		//set current position
		position = *startPos;
		this->startPos = *startPos;			
		unitVelocity.set(0,0,1);		
	}
	~EvilChopper(void)
	{
		
	}

	void drawTorso()
	{
		bottom.draw();
		mid.draw();
		
		glPushMatrix();
		glScalef(1,-1,1);
		top.draw();
		glPopMatrix();
		
		glPushMatrix();
			glScalef(1,-1,1);
			glDisable(GL_TEXTURE_2D);
			glColor4f(0.6, 0.6, 0.9, 0.9);
			window.draw();
			glEnable(GL_TEXTURE_2D);
		glPopMatrix();
	}
	
	void drawBlades(float x)
	{
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glVertex2f(-1*x, 0.1);
			glVertex2f(-1*x, -0.1);
			glVertex2f(x, -0.1);
			glVertex2f(x, 0.1);
			
			glVertex2f(-0.1, -1*x);
			glVertex2f(0.1, -1*x);
			glVertex2f(0.1, x);
			glVertex2f(-0.1, x);
		glEnd();
	}
	
	void drawArm()
	{
		arm.draw();
		
		gunarm.draw();
		
		glPushMatrix();
			glTranslatef(-0.55, -0.37, 0);
			gluCylinder(quad, 0.1, 0.1, 1, 10, 3);
		glPopMatrix();
	}
	
	void drawTail()
	{
		glColor4f(0.2, 0.2, 0.2, 1);
		tailBox.draw();
		
		glPushMatrix();
			glScalef(1,1,-1);
			gluCylinder(quad, 0.3, 0.1, 5.0, 10, 10);
			glTranslatef(0,0,3);
			gluDisk(quad, 0, 0.1, 10, 10);
		glPopMatrix();
		//draw tail rotor
		glPushMatrix();
			glTranslatef(0.15, 0, -4.8);
			glRotatef(-90, 0, 1, 0);
			glRotatef(rot_angle, 0, 0, 1);
			glColor4f(0.2, 0.2, 0.2, 0.7);
			glDisable(GL_TEXTURE_2D);
			drawBlades(1);
			glEnable(GL_TEXTURE_2D);
		glPopMatrix();
	}
	
	void drawTopRotors()
	{
		glPushMatrix();
			glRotatef(-90, 1, 0, 0);
			gluCylinder(quad, 0.05, 0.05, 0.5, 10, 3);
			glTranslatef(0,0,0.5);
			gluDisk(quad, 0, 0.05, 10, 10);
			glTranslatef(0,0,-0.1);
			glColor4f(0.2,0.2,0.2,0.7);
			glRotatef(rot_angle, 0, 0, 1);
			glDisable(GL_TEXTURE_2D);
			drawBlades(4);
			glEnable(GL_TEXTURE_2D);
		glPopMatrix();
	}

	
	void drawBody()
	{
	//	Arrow tempPos= position;
	//	tempPos.setY(position.getY() + 50);
	//	DrawArrow(unitVelocity.getCoords(),tempPos.getCoords(),1000,1,0,0);		
		
		//SETUP
		glPushMatrix();		
		
		//tranlate to right position
		glTranslatef(position.getX(),position.getY(),position.getZ());	
		glColor3f(0,1,0);
		//glutWireSphere(this->collisionInfo.boundingRadius,24,24);
		
		glRotatef(calculateXZTurnAngle() ,0,1,0);//rotate to proper local orientation
		
		//DO ACTUAL DRAWING	
		glPushMatrix();
		glScalef(4,4,4);
		glTranslatef(0,-50,100);
		glScalef(25,25,25);		
		glRotatef(30,1,0,0);		
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glBindTexture(GL_TEXTURE_2D, texname);
		
		glColor3f(0.7, 1, 0.2);
		drawTorso();
		
		glColor3f(0.7, 1, 0.2);
		glPushMatrix(); //tail end
			glTranslatef(0, 0.0, -3.5);
			drawTail();
		glPopMatrix();
		
		glColor3f(0.7, 1, 0.2);
		glPushMatrix(); //top blades
			glTranslatef(0, 1, -1.8);
			drawTopRotors();
		glPopMatrix();
		
		glColor3f(0.7, 1, 0.2);
		glPushMatrix(); //arms
			glTranslatef(-0.5, 0, -2);
			if(hitPoints <= 75)
			{
				smoke.draw();
				smoke.update();
				if(hitPoints <= 25)
				{
					fire.draw();
					fire.update();
				}
			}
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, texname);
			glColor3f(0.7, 1, 0.2);
			drawArm();
			glTranslatef(1, 0, 0);
			glScalef(-1,1,1);
			drawArm();
		glPopMatrix();
	
// 	if(picked)
// 	{
// 		glColor4f(0,1,0,0.1);
// 		glDisable(GL_TEXTURE_2D);
// 		glDisable(GL_LIGHTING);
// 		glPushMatrix();
// 			glTranslatef(0,(-1+1.5)/2,-7.5/2);
// 			glScalef(1.5, 1.25, 4.25);
// 			glColor4f(0,1,0,0.5);
// 			glutWireCube(2);
// 		glPopMatrix();
// 		picked = false;
// 	}
		glPopMatrix(); //pop the scale
		
		glPopMatrix();
		glDisable(GL_TEXTURE_2D);
		
	}
	double calculateXZTurnAngle()
	{
		Arrow dir2d = unitVelocity;
		dir2d.setY(0);
		dir2d.setLength(1);			
		return zAxis.angleBetween(&dir2d,&yAxis);
	}
	void update()
	{
		if(hitPoints <= 0)
		{
			position = startPos;
			hitPoints = 100;
			score += 100;
			fire.reset();
			smoke.reset();
		}
		
		rot_angle+= 57;
		if(rot_angle >= 360.0) rot_angle = 0.0 + (rot_angle - 36.0);
		double y;	
		double x;
		double z;	
		x = position.getX();
		z = position.getZ();
		y = position.getY();
		
		Arrow wantedDir = pHero->position.sub(&position);	
		Arrow t = wantedDir;
		t.setY(0);
		double wantedDistance = wantedDir.getLength();
		
		double wanted2DDist = t.getLength();
		double distanceX = wantedDir.getX();
		double distanceY = wantedDir.getY();
		double distanceZ = wantedDir.getZ();
		double yVel = unitVelocity.getY();
		double xVel = unitVelocity.getX();
		double zVel = unitVelocity.getZ();

		Arrow currentXZ = unitVelocity;
		currentXZ.setY(0);
		currentXZ.setLength(1);	
		Arrow wantedDir2d(distanceX,0,distanceZ);			
		wantedDir2d.setLength(1);
		double angleDiff = wantedDir2d.angleBetween(&currentXZ,&yAxis);
		if(absD(angleDiff) > 2)
		{
			double increment;
			if(angleDiff < 0)
				increment = min(3 ,angleDiff * -1);
			else
				increment = max(-3,angleDiff* -1);
			unitVelocity = unitVelocity.rotateInY(increment);		

		}
		if(wanted2DDist > 12000)
		{
			//if distance is too big make them go up until around 20000
			speed = min(speed + 1,maxSpeed); //		
			if(position.getY() < 25000)
			{
				unitVelocity.setY(min(unitVelocity.getY() + 0.1,0.9));
				unitVelocity.setLength(1);							
			}
			else
			{
				unitVelocity.setY(max(unitVelocity.getY() - 0.1,-0.9));
				unitVelocity.setLength(1);							
			}
		}
		else
		{
			if(wanted2DDist < 4000)
			{
				if(absD(distanceY) <  2)
					speed = max(speed - 1,-maxSpeed);		
				unitVelocity.setY(max(unitVelocity.getY() - 0.1,-0.9));
				unitVelocity.setLength(1);
			}
			else
			{
				//right distance check height diff and shoot
				//speed = speed * 0.8;				
				if(absD(distanceY) < 600 && timer.check())
				{
					Arrow unitWanted = wantedDir;
					unitWanted.setLength(1);
					mmanager->launch(&position,&unitWanted,300,this);
				}
			}
			
			if(distanceY > 2)
			{
				unitVelocity.setY(min(unitVelocity.getY() + 0.1,0.9));
				unitVelocity.setLength(1);
				speed = min(speed + 3,maxSpeed);		
			}
			else if (distanceY < -2)
			{
				unitVelocity.setY(max(unitVelocity.getY() - 0.1,-0.9));
				unitVelocity.setLength(1);						
				speed = min(speed + 3,maxSpeed);		
	
			}
			else
			{
				unitVelocity.setY(max(unitVelocity.getY() - 0.3,0))	;			
				unitVelocity.setLength(1);						
				speed = speed * 0.6;		
						
			}
		}
			
			
		
		//check for collision
		pCollisionManager->RunCollisionsCheck(this);
		if(collisionInfo.collisionFound)
		{
 			x = collisionInfo.lastGoodPoint.getX();
			z = collisionInfo.lastGoodPoint.getZ();
			y = collisionInfo.lastGoodPoint.getY();
			
			unitVelocity = collisionInfo.slideDirection;					
			unitVelocity.setLength(1);	
			speed = speed * -0.6;
			//collisionInfo.collisionFound = false;
		}
		else
		{
			x+= unitVelocity.getX() * speed;
			z+= unitVelocity.getZ() * speed;
			y += unitVelocity.getY() * speed;
		}			
		position.set(x,y,z);	
	}

};



