#include "common.h"
#include "geometry.cpp"
#include "object.cpp"
#include "custombox.cpp"
#include "terrain.cpp"
#include "texture.h"
#include "hero.cpp"
#include "collisionmanager.h"
#include "smoke.h"
#include "fire.h"
#include "missilemanager.h"
#include "mytime.h"

extern CollisionManager gCollisionManager;
extern TEXTURE camoTex;
extern TEXTURE metalTex;
extern TEXTURE wheelTex;

class EvilTank : public Object
{
public:
	Terrain * pTerrain;
	Hero * pHero;
	CollisionManager * pCollisionManager;
	
	GLUquadric* tankQuadric;
	double wheelSpinAngle;
	CustomBox tankTop;
	double wantedX;
	double wantedZ;
	Arrow baseAxis;
	double turnAngle;
	Fire fire;
	Smoke smoke;
	MissileManager * mmanager;
	MyTime timer;
	EvilTank(void)
	{
		isFlying = false;
		this->collisionInfo.boundingRadius = 400;
		maxSpeed = 150;
		isEnemy = true;
		hitPoints = 100;
		timer = MyTime(2);
	}
	void init(Terrain * terrain,Hero * hero,CollisionManager * collisionManager,Vertex * startPos,
	MissileManager * mmanager
	)
	{
		this->pTerrain = terrain;	
		this->pHero = hero;
		this->mmanager = mmanager;
		tankQuadric =  gluNewQuadric();
		pCollisionManager = collisionManager;
		Corner v1,v2,v3,v4,v5,v6,v7,v8;
		v1.set(-5,6,-9);
		v2.set(5,6,-9);
		v3.set(5,6,9);
		v4.set(-5,6,9);
		
		v5.set(-8,0,-12);
		v6.set(8,0,-12);
		v7.set(8,0,12);
		v8.set(-8,0,12);
		tankTop.set(&v1,&v2,&v3,&v4,&v5,&v6,&v7,&v8);
		wheelSpinAngle = 0;
		//set current position
		double yPos;
		double xPos = startPos->getX();
		double zPos = startPos->getZ();
		pTerrain->getHeightAndNormal(xPos,zPos,&yPos,&baseAxis);
		position.set(xPos,yPos,zPos); 
		this->startPos = *startPos;		
		turnAngle = 0;
		
	}
	~EvilTank(void)
	{
		
	}

	void drawBody()
	{
		//Arrow tempPos= position;
		Vertex origin(0,0,0);
		//tempPos.setY(position.getY() + 50);
		//DrawArrow(unitVelocity.getCoords(),tempPos.getCoords(),1000,1,0,0);
		//DrawArrow(baseAxis.getCoords(),position.getCoords(),1000,0,0,1);
		//SETUP
		glPushMatrix();		
		
		//tranlate to right position
		glTranslatef(position.getX(),position.getY(),position.getZ());		
		glColor3f(0,1,0);
		//glutWireSphere(this->collisionInfo.boundingRadius,24,24);
		
		//rotate to proper orientation .assume default is 0,1,0
   		baseAxis.setLength(1);		
		Arrow axis = yAxis.cross(baseAxis);		
		double angle = yAxis.angleBetween(&baseAxis,0);	
		glRotatef(angle,axis.getX(),axis.getY(),axis.getZ());

		//rotate by rotation angle
		//DrawArrow(yAxis.getCoords(),origin.getCoords(),1000,0,1,0);		
		glRotatef(turnAngle ,0,1,0);//rotate to proper local orientation
		
		//DO ACTUAL DRAWING	
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D,camoTex.TexID);		
		gluQuadricTexture(tankQuadric, GL_TRUE);
		gluQuadricNormals(tankQuadric, GL_SMOOTH);
	
      		glColor3f(0.4,0.35,0.35);
		glScalef(12,12,12);
		glTranslatef(0,7,0);		
		
		DrawTexturedBox(26,9,40);	//main tank box
	
		//draw left side of tank		
		glPushMatrix();		
		glTranslatef(17,0,20);
		for(int	i =0;i < 4;i++)
		{
			glBindTexture(GL_TEXTURE_2D,camoTex.TexID);		
	        glColor3f(0.25,0.25,0.25);
			glPushMatrix();
			glRotatef(90,0,1,0);
			glTranslatef(0,0,-6.5);
			glRotatef(wheelSpinAngle,0,0,1);			
			gluCylinder(tankQuadric,6.7,6.7,6.5,24,24);						
	        glColor3f(0.6,0.6,0.6);
			glBindTexture(GL_TEXTURE_2D,wheelTex.TexID);		
			gluDisk(tankQuadric,0,6.7,24,24);
			glTranslatef(0,0,6.5);
			gluDisk(tankQuadric,0,6.7,24,24);					
			glPopMatrix();
			glTranslatef(0,0,-13);
		}			
			glPopMatrix();
		//draw right side of tank
		
		glPushMatrix();
		glTranslatef(-17,0,20);
		for(int	i = 4;i < 8;i++)
		{
			glBindTexture(GL_TEXTURE_2D,camoTex.TexID);		
	          	glColor3f(0.25,0.25,0.25);
			glPushMatrix();
			glRotatef(90,0,1,0);
			glRotatef(wheelSpinAngle,0,0,1);
			gluCylinder(tankQuadric,6.7,6.7,6.5,24,24);						
	        glColor3f(0.6,0.6,0.6);
			glBindTexture(GL_TEXTURE_2D,wheelTex.TexID);		
			gluDisk(tankQuadric,0,6.7,24,24);
			glTranslatef(0,0,6.5);
			gluDisk(tankQuadric,0,6.7,24,24);			
			glPopMatrix();
			glTranslatef(0,0,-13);
		}			
		glPopMatrix();	
		glBindTexture(GL_TEXTURE_2D,camoTex.TexID);		
		
		//draw left top	
		glColor3f(0.8,0.8,0.2);
		glPushMatrix();
		glTranslatef(14,7.1,0);
		glPushMatrix();
			glScalef(10,10,10);
			if(hitPoints <= 75)
			{
				smoke.draw();
				smoke.update();
				if(hitPoints <= 25)
				{
					fire.draw();
					fire.update();
				}
			}
		glEnable(GL_TEXTURE_2D);		
		glPopMatrix();
		glColor3f(0.8,0.8,0.2);
		glBindTexture(GL_TEXTURE_2D,camoTex.TexID);
		DrawTexturedBox(6.5,0.8,44);		
		
		glPushMatrix();
		glTranslatef(0,0,-22);
		glRotatef(-40,1,0,0);
		glTranslatef(0,0,-4);
		DrawTexturedBox(6.5,0.8,8);		
		glPopMatrix();
		
		glPushMatrix();
		glTranslatef(0,0,22);
		glRotatef(220,1,0,0);
		glTranslatef(0,0,-4);
		DrawTexturedBox(6.5,0.8,8);		
		glPopMatrix();
		glPopMatrix();
		
		//draw right top	
		glColor3f(0.8,0.8,0.2);
		glPushMatrix();
		glTranslatef(-14,7.1,0);
		DrawTexturedBox(6.5,0.8,44);		
		
		glPushMatrix();
		glTranslatef(0,0,-22);
		glRotatef(-40,1,0,0);
		glTranslatef(0,0,-4);
		DrawTexturedBox(6.5,0.8,8);		
		glPopMatrix();
		
		glPushMatrix();
		glTranslatef(0,0,22);
		glRotatef(220,1,0,0);
		glTranslatef(0,0,-4);
		DrawTexturedBox(6.5,0.8,8);		
		glPopMatrix();							
		glPopMatrix();
		//draw pivot cylinder
		glTranslatef(0,4.6,0);
		glPushMatrix();
		glRotatef(-90,1,0,0);	
		glColor3f(0.7,0.4,0.4);	
		gluCylinder(tankQuadric,8.2,8.2,3,24,24);		
		glPopMatrix();		
		glTranslatef(0,3,0);
		
		//Draw top box with turret	
		glPushMatrix();
		glColor3f(0.8,0.8,0.8);		
		tankTop.draw();
		glPushMatrix();
		glTranslatef(0,4,0);
		//glRotatef(-90,0,1,0);		
		glColor3f(0.3,0.4,0.3);
		gluCylinder(tankQuadric,1.0,1.0,29,24,24);
		glTranslatef(0,0,24);
		gluDisk(tankQuadric,0.8,1.5,29,24);				
		gluCylinder(tankQuadric,1.5,1.5,6,24,24);
		glTranslatef(0,0,6);
		gluDisk(tankQuadric,1.0,1.5,24,24);		
		glPopMatrix();
		//draw top lid
		glTranslatef(-2,6,-1);
		glRotatef(-90,1,0,0);
		glColor3f(0.4,0.4,0.4);
		gluCylinder(tankQuadric,2.5,2.5,1,24,24);
		glTranslatef(0,0,1);
		glBindTexture(GL_TEXTURE_2D,metalTex.TexID);		
		gluDisk(tankQuadric,0,2.5,29,24);				
		glPopMatrix();
		
		//
		
		glPopMatrix();
		glDisable(GL_TEXTURE_2D);
		
	}
	void updateUnitVelocity()
	{
		Arrow dir2d = zAxis.rotateInY(turnAngle);
		Arrow w = dir2d.cross(baseAxis);
		unitVelocity = baseAxis.cross(w);
		unitVelocity.setLength(1);		

	}
	void updateTurnAngle()
	{
		Arrow dir2d = unitVelocity;
		dir2d.setY(0);
		dir2d.setLength(1);
		turnAngle = zAxis.angleBetween(&unitVelocity,&yAxis);
						
	}
	void update()
	{
		if(hitPoints <= 0)
		{
			position = startPos;
			hitPoints = 100;
			score += 100;
			fire.reset();
			smoke.reset();
		}
		
		double y;	
		double x;
		double z;	
		x = position.getX();
		z = position.getZ();
		y = position.getY();
		//now calculate new direction in 2d
		Arrow wantedDir(pHero->position.getX() - x,0,pHero->position.getZ() - z);
		Vertex shootPos= position;		
		shootPos.setY(shootPos.getY() + 100);		
		Arrow shootingArrow = pHero->position.sub(&shootPos);
		double shootingHeightDiff = shootingArrow.getY();
		shootingArrow.setLength(1);
		double wantedDistance = wantedDir.getLength();
		Arrow current2d = zAxis.rotateInY(turnAngle);	
		current2d.setLength(1);
		baseAxis.setLength(1);
		wantedDir.setLength(1);
		double angleDiff = wantedDir.angleBetween(&current2d,&yAxis);
		if(absD(angleDiff) > 2)
		{
			speed = speed * 0.95;
			if(angleDiff < 0)
				turnAngle += min(2,angleDiff * -1);
			else
				turnAngle += max(-2,angleDiff* -1);
				
			wheelSpinAngle+= 1;
			updateUnitVelocity();
		}			
		else	
		{
			if(wantedDistance > 12000 || wantedDistance < 8000 )
			{	
				if(wantedDistance > 12000)
				{
					speed = min(speed + 1,maxSpeed);
					wheelSpinAngle+= 2;
				}
				else
				{
					speed = max(speed - 1,-maxSpeed);
					wheelSpinAngle+= 2;
				}
				//check for collision
				pCollisionManager->RunCollisionsCheck(this);
				if(collisionInfo.collisionFound)
				{
					x = collisionInfo.lastGoodPoint.getX();
					z = collisionInfo.lastGoodPoint.getZ();
					unitVelocity = collisionInfo.slideDirection;					
					unitVelocity.setLength(1);	
					updateTurnAngle();		
					speed = speed * -0.6;
					//collisionInfo.collisionFound = false;
				}
				else
				{
					x+= unitVelocity.getX() * speed;
					z+= unitVelocity.getZ() * speed;
				}
			}
			else 
			{
				speed *= 0.8;
				if(timer.check() && absD(shootingHeightDiff) < 4000 )
				{
					mmanager->launch(&shootPos,&shootingArrow,300,this);
				}
			}
		}
		//decided on position		
		pTerrain->getHeightAndNormal(x,z,&y,&baseAxis);
		position.set(x,y,z);	
		//done		
	}
};




