#include "explosion.h"

Explosion::Explosion( int spread )
{
	this->spread = spread;
	texture = BlobTexture::getBlobTexture();
	reset();
}

Explosion::~Explosion()
{

}

void Explosion::reset()
{
	static float updown = 1.0;
	for(int i = 0; i < NUMPARTICLES; i++)
	{
		Particle* p = &particles[i];
		p->alive = true;
		p->life = 1.0;
		p->fade = (random() % 10) / 100.0 + 0.03;
		//double gval = (random() % 255) / 255.0;
		p->color.set(1.0, 0.0, 0.0);
		p->position.set(0.01,0.01,0.01);
		
		int angle = random() % spread;
		int rotation = random() % 360;
		double anglef = (angle/180.0)*M_PI;
		double rotf = (rotation/180.0)*M_PI;
		double length = 1.0*tan(anglef);
		double x = length*cos(rotf);
		double z = length*sin(rotf);
		Vertex xz(x, 0, z);
		Vertex v = Vertex(0,1,0).add( &xz );
		v.setLength(2.0*updown);
		p->velocity.set(&v);
		
		Vertex old = p->velocity.mul( -10 );
		p->oldpos.set(&old);
		
		Vertex a = v.mul(-1*p->fade);
		p->accel.set(&a);
		
		updown *= -1.0;
	}
	
	for(int i = 0; i < NUMBLOBS; i++)
	{
		Fuzzy* f = &fuzzies[i];
		f->alive = true;
		f->life = 1.0;
		f->rotation = float(random() % 360);
		f->fade = (random() % 10) / 100.0 + 0.03;
		f->pos.set(0,0,0);
		
		int angle = random() % (int)(1.5 * spread);
		int rotation = random() % 360;
		double anglef = (angle/180.0)*M_PI;
		double rotf = (rotation/180.0)*M_PI;
		double length = 1.0*tan(anglef);
		double x = length*cos(rotf);
		double z = length*sin(rotf);
		Vertex xz(x, 0, z);
		Vertex v = Vertex(0,1,0).add( &xz );
		v.setLength(2.0*updown);
		f->velocity.set(&v);
		
		Vertex a = v.mul(-1*f->fade);
		f->accel.set(&a);
		
		updown *= -1.0;
	}
	
}

bool Explosion::draw()
{
	//glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
	//glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glDepthMask(GL_FALSE);
	bool anyalive = false;
	glBegin(GL_LINES);
	for(int i = 0; i < NUMPARTICLES; i++)
	{
		if(particles[i].alive)
		{
			Particle* p = &particles[i];
			
			glColor4d(p->color.coords[0], p->life, p->color.coords[2], p->life);
			
			Vertex old = p->position.sub( &p->oldpos );
			glVertex3dv(old.getCoords());
			glVertex3dv(p->position.getCoords());
			
			p->life -= p->fade;
			if(p->life <= 0.0)
			{
				p->alive = false;
			}
			else
			{
				anyalive = true;
				
				Vertex newpos( p->position.add(&p->velocity) );
				p->position.set(&newpos);
			
				Vertex newvel( p->velocity.add( &p->accel) );
				p->velocity.set(&newvel);
			}
		}
	}
	glEnd();
	
	glEnable(GL_TEXTURE_2D);
	texture->bind();
	for(int i = 0; i < NUMBLOBS; i++)
	{
		Fuzzy* f = &fuzzies[i];
		glPushMatrix();
			glRotatef(f->rotation, 0, 1, 0);
			glColor4f(1.0, f->life, 0.0, f->life);
			glBegin(GL_TRIANGLE_STRIP);
				static const float radius = 10;
				double x = f->pos.getX();
				double y = f->pos.getY();
				double z = f->pos.getZ();
				glTexCoord2f(1,1);
				glVertex3d(x+radius, y+radius, z);
				glTexCoord2f(0,1);
				glVertex3d(x-radius, y+radius, z);
				glTexCoord2f(1,0);
				glVertex3d(x+radius, y-radius, z);
				glTexCoord2f(0,0);
				glVertex3d(x-radius, y-radius, z);
			glEnd();
		glPopMatrix();
		
		f->life -= f->fade;
		if( f->life <= 0.0 )
			f->alive = false;
		else
		{
			anyalive = true;
			
			Vertex newpos( f->pos.add(&f->velocity) );
			f->pos.set(&newpos);
		
			Vertex newvel( f->velocity.add( &f->accel) );
			f->velocity.set(&newvel);
		}
	}
	
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glDepthMask(GL_TRUE);
	//glEnable(GL_DEPTH_TEST);
	//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	return anyalive;
}
