#include "fire.h"

Fire::Fire(bool points)
{
	for(int i = 0; i < NUM_BLOBS; i++)
	{
		newBlob(i);
	}
	if(!points)
		texture = BlobTexture::getBlobTexture();
	this->points = points;
}

Fire::~Fire()
{

}

void Fire::draw()
{
	glDepthMask(GL_FALSE);
	if(!points)
	{
		glEnable(GL_TEXTURE_2D);
		texture->bind();
	}
	else
		glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	for(int i = 0; i < NUM_BLOBS; i++)
	{
		Blob* b = &blobs[i];
		double y = b->pos.getY();
		glPushMatrix();
			glRotatef(b->rotation, 0,1,0);
			float gval = 1.0 - (b->pos.getY() / 4.0);
			GLfloat colors[] = { 1, gval, gval*0.5, gval };
			if(!points)
			{
				glBegin(GL_TRIANGLE_STRIP);
					static const float radius = 0.5;
					glColor4fv(colors);
					double x = b->pos.getX();
					double z = b->pos.getZ();
					glTexCoord2f(1,1);
					glVertex3d(x+radius, y+radius, z);
					glTexCoord2f(0,1);
					glVertex3d(x-radius, y+radius, z);
					glTexCoord2f(1,0);
					glVertex3d(x+radius, y-radius, z);
					glTexCoord2f(0,0);
					glVertex3d(x-radius, y-radius, z);
				glEnd();
			}
			else
			{
				glBegin(GL_POINTS);
					glColor4fv(colors);
					glVertex3dv(b->pos.getCoords());
				glEnd();
			}
		glPopMatrix();
	}
	glEnable(GL_LIGHTING);
	glDepthMask(GL_TRUE);
	glDisable(GL_TEXTURE_2D);
}

void Fire::update()
{
	for(int i = 0; i < NUM_BLOBS; i++)
	{
		Blob* b = &blobs[i];
		double y = b->pos.getY() + b->speed;
		if( y >= 4.0 )
		{
			newBlob(i);
		}
		else //update
		{
			b->pos.setY( y );
			int col = b->col;
			int newcol = int((y / 4.0) * 255.0);
			if( col < newcol )
			{
				float diff = noise[b->row][newcol] - noise[b->row][col];
				double z = b->pos.getZ();
				b->pos.setZ( z + diff );
				b->col = newcol;
			}
		}
	}
}

void Fire::reset()
{
	for(int i = 0; i < NUM_BLOBS; i++)
	{
		newBlob(i);
	}
}

void Fire::newBlob(int i)
{
	blobs[i].rotation = float(random() % 360);
	blobs[i].row = random() % 256;
	blobs[i].col = 0;
	blobs[i].pos.setX( random() % 100 / 100.0 );
	blobs[i].pos.setY( 0 );
	blobs[i].pos.setZ( random() % 100 / 100.0 + noise[blobs[i].row][0] );
	blobs[i].speed = (random() % 100) / 1000.0 + 0.003;
}

