#ifndef HERO_CPP
#define HERO_CPP

#include "common.h"
#include "cockpit.h"
#include "object.cpp"
#include "collisionmanager.h"
#include "missilemanager.h"

extern TEXTURE metalTex;
extern TEXTURE panelTex;

class Hero : public Object
{
	public:
	Arrow  upAxis;
	Vertex lookAtPoint;
	Cockpit cockpit;
	CollisionManager * pCollisionManager;
	bool foundHit;
	Vertex hitTarget;
	GLint viewport[4];
	double proj[16];
	double model[16];
	double xAimOffset;
	double yAimOffset;
	GLUquadric* turretQuadric;
	double turretUpMultiplier;
	double turretForwMultiplier;
	MissileManager* missle;
	Arrow shootingVector;
	Hero()
	{
		isFlying = true;
		isHero = true;
		unitVelocity.set(1,0,-1);
		shootingVector = unitVelocity;
		upAxis.set(0,1,0);
		this->updateLookAtPoint();
		speed = 0;
		foundHit = false;
		xAimOffset=0;
		yAimOffset =0;
		turretUpMultiplier = 7;
		turretForwMultiplier = 15;
		maxSpeed = 200;
		hitPoints = 100;
		collisionInfo.boundingRadius = 45;
	}
	void init(CollisionManager * collisionManager,Vertex * startPos, MissileManager* mm)
	{
		position = *startPos;
		this->startPos = *startPos;
		pCollisionManager = collisionManager;
		turretQuadric = gluNewQuadric();
		gluQuadricTexture(turretQuadric, GL_TRUE);
		cockpit.init(this);
		missle = mm;
	}	
	~Hero()
	{
		gluDeleteQuadric(turretQuadric);
	
	}
	void changeSpeed(bool increase)
	{
		if(increase)
			speed = min(speed + 7,maxSpeed);
		else
			speed = max(speed - 7,-maxSpeed);
	}
	void changePitch(bool up,double multiplier)
	{	
		Arrow tempUpVector;
		Arrow w = upAxis.cross(unitVelocity);
		tempUpVector.set(&upAxis);
		if(!up)
			tempUpVector = tempUpVector.mul(-1);		

		Arrow diff = tempUpVector.sub(&unitVelocity);
		diff = diff.mul(multiplier); 
		unitVelocity = unitVelocity.add(&diff);
		upAxis = unitVelocity.cross(w);
		unitVelocity.setLength(1);
		updateLookAtPoint();
		upAxis.setLength(1);
	}
	void changeRoll(bool right,double multiplier)
	{
		Arrow w = upAxis.cross(unitVelocity);
		if(!right)
			w = w.mul(-1);		

		Arrow diff = w.sub(&upAxis);
		diff = diff.mul(multiplier); 
		upAxis = upAxis.add(&diff);
		upAxis.setLength(1);
	}


	void changePan(bool left,double multiplier)
	{
		
		Arrow w = upAxis.cross(unitVelocity);
		if(!left)
			w = w.mul(-1);
		w.setLength(1);

		Arrow diff = w.sub(&unitVelocity);
		diff = diff.mul(multiplier); 
		unitVelocity = unitVelocity.add(&diff);
		unitVelocity.setLength(1);
		updateLookAtPoint();
	}

	void drawBody()
	{
	     glPushMatrix();
	     glTranslatef(0,0,-5);
	     cockpit.draw();
	     Vertex pos(0,0,0);
             glPopMatrix();
	}
	void updateLookAtPoint()
	{
		lookAtPoint = position.add(&unitVelocity);			
	}
	void fireMissle()
	{
		Vertex misslePos = position;
		Arrow t = upAxis.mul(turretUpMultiplier);
		misslePos =misslePos.add(&t);
		t = unitVelocity.mul(turretForwMultiplier);
		misslePos =misslePos.add(&t);
		missle->launch(&misslePos,&shootingVector,400,this);
		
		/*shootingVector.print();
		unitVelocity.print(); printf("\n");
		*/
	}
	void changeAim(double xCoord,double yCoord)
	{
			xAimOffset =xCoord;
			yAimOffset =yCoord;

	}
	void drawShootingVector()
	{

		glGetIntegerv(GL_VIEWPORT,viewport);
		glGetDoublev(GL_PROJECTION_MATRIX,proj);
		glGetDoublev(GL_MODELVIEW_MATRIX,model);
		double objx,objy,objz;
		double objx2,objy2,objz2;
		
		int res = gluUnProject(xAimOffset,yAimOffset,0,model,proj,viewport,&objx,&objy,&objz);
		res = gluUnProject(xAimOffset,yAimOffset,1,model,proj,viewport,&objx2,&objy2,&objz2);
		Vertex start(objx,objy,objz);
		Vertex end(objx2,objy2,objz2);
		Vertex turretPos = position;
		Arrow upOffset = upAxis.mul(turretUpMultiplier);
		Arrow forwOffset = unitVelocity.mul(turretForwMultiplier);
		turretPos = turretPos.add(&upOffset);
		turretPos = turretPos.add(&forwOffset);
		shootingVector = end.sub(&turretPos);
		//check for collision and change cockpit params
		Object * hitObject =0;
		Vertex resultPoint;
		if(pCollisionManager->findObjectHit(start, shootingVector ,&resultPoint,&hitObject,this)		
		&& hitObject->isEnemy)
		{	
			cockpit.enemyInTarget =true;
		}
		else
		{
			cockpit.enemyInTarget =false;		
		}
		
		
		//DrawArrow(shootingVector.getCoords(),turretPos.getCoords(),1,1,0,0);
		shootingVector.setLength(1);
		//shootingVector.print();
		double theta;
		Arrow axis = shootingVector.cross(zAxis);
		
		theta =shootingVector.angleBetween(&zAxis,&axis) * -1;
		Arrow w = unitVelocity.cross(zAxis);			
		double rotCorrection = upAxis.angleBetween(&yAxis,&zAxis);
		
		
		glColor3f(0.9,0.9,0.9);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, metalTex.TexID);
		
		glPushMatrix();
		glTranslatef(turretPos.getX(),turretPos.getY(),turretPos.getZ());
		glTranslatef(upOffset.getX(),upOffset.getY(),upOffset.getZ());
		glTranslatef(forwOffset.getX(),forwOffset.getY(),forwOffset.getZ());

		glRotatef(theta,axis.getX(),axis.getY(),axis.getZ());
		glRotatef(rotCorrection,0,0,1);
		DrawTexturedBox(4,4,15);
		gluCylinder(turretQuadric,1.5,1.5,24,24,24);
	
		glPopMatrix();	
		glDisable(GL_TEXTURE_2D);			
	}
	void update()
	{
		if(hitPoints <= 0)
		{
			printf("You died. %d points.\n", score);
			exit(1);
		}
		
		if(speed < -0.1 || speed > 0.1)
		{
			unitVelocity.setLength(1);
    			Arrow increment = unitVelocity.mul(speed);
			pCollisionManager->RunCollisionsCheck(this);
 			if(collisionInfo.collisionFound)
			{
 				position = collisionInfo.lastGoodPoint;
				unitVelocity = collisionInfo.slideDirection;
				speed = speed * 0.3;
				updateLookAtPoint();
				return;
			}
			else
			{
				position = position.add(&increment);
				updateLookAtPoint();
			}
			if(speed < -0)
			{
					speed += sqrt(sqrt(speed * -1));
			}
			else if(speed > 0)
			{
					speed -= sqrt(sqrt(speed));
			}	
		}
		else
			updateLookAtPoint();

	}

};
#endif



