#include "karm.h"

KArm::KArm()
{
	angle = min_arm_angle;
	delta = (max_arm_angle - min_arm_angle)/15;
	counter = 0;
	animating = false;
}

void KArm::draw(bool smoothly)
{
	if(smoothly && animating)
	{
		if( counter == 0 )
		{
			if(angle == min_arm_angle)
			{
				if(delta < 0) delta *= -1.0;
			}
			else
				if(delta > 0) delta *= -1.0;
			angle += delta;
			counter++;
		}
		else if( counter < 15 )
		{
			angle += delta;
			counter++;
		}
		else
		{
			if( delta < 0 )
				angle = min_arm_angle;
			else
				angle = max_arm_angle;
			counter = 0;
			animating = false;
		}
	}
	// the shoulder is more like an upper arm, really
	const float shoulder_x = 0.1;
	const float shoulder_y = 0.2;
	const float shoulder_z = 0.1;
	const float arm_x = 0.4;
	const float arm_y = 0.1;
	const float arm_z = 0.1;
	const float paw_x = 0.1, paw_y = 0.1, paw_z = 0.1;
	glPushMatrix();
		glRotatef(angle, 0.0, 0.0, 1.0);
		glTranslatef(0.0, -0.5*shoulder_y, 0.0);
		drawScaleCube(shoulder_x, shoulder_y, shoulder_z); //shoulder
		glTranslatef(-0.5*arm_x, -0.5*shoulder_y, 0.0); //move into place for arm
		drawScaleCube(arm_x, arm_y, arm_z); //arm
		glTranslatef(-0.5*arm_x+0.5*paw_x, -0.5*paw_y, 0.0); //move into place for paw
		drawScaleCube(paw_x, paw_y, paw_z); //paw
	glPopMatrix();
}

void KArm::toggle(bool smoothly)
{
	if( !smoothly )
	{
		if(angle == min_arm_angle)
			angle = max_arm_angle;
		else
			angle = min_arm_angle;
	}
	else
	{
		if( !animating )
			animating = true;
	}
}
