#include "khead.h"

KHead::KHead()
{
	angle = 0.0;
	head_pos = 1;
	animating = moving_from_left = false;
	counter = 0;
	delta = (max_swivel - min_swivel)/15/2;
	ear_angle = 0.0;
}

void KHead::draw(bool smoothly)
{
	if( smoothly && animating )
	{
		if( counter == 0 )
		{
			switch(head_pos)
			{
				case 0:
					if( delta < 0 ) delta *= -1;
					break;
				case 1:
					if( moving_from_left && delta < 0 ) delta *= -1;
					else if( !moving_from_left && delta > 0 ) delta *= -1;
					break;
				case 2:
					if( delta > 0 ) delta *= -1;
					break;
			}
			angle += delta;
			counter++;
		}
		else if( counter < 15 )
		{
			angle += delta;
			counter++;
		}
		else
		{
			switch( head_pos )
			{
				case 0:
					angle = 0.0;
					head_pos = 1;
					break;
				case 1:
					if( moving_from_left )
					{
						angle = max_swivel;
						moving_from_left = false;
						head_pos = 2;
					}
					else
					{
						angle = min_swivel;
						moving_from_left = true;
						head_pos = 0;
					}
					break;
				case 2:
					angle = 0.0;
					head_pos = 1;
					break;
			}
			counter = 0;
			animating = false;
		}
	}
	glPushMatrix();
		glTranslatef(0.0, 0.1, 0.0); //nudge it into place
		drawScaleCube(0.3,0.7,0.3); // neck
		glTranslatef(0.0, 0.5*0.7, 0.0); // move to top of neck
		glRotatef(10.0, 0.0, 0.0, 1.0); // tilt forward slightly
		glRotatef(angle, 0.0, 1.0, 0.0); //turn head
		glTranslatef(-0.05, 0.0, 0.0); // nudge
		drawScaleCube(0.4, 0.3, 0.4); // head
		glPushMatrix();
			glTranslatef(-0.4*0.5, 0.0, 0.0);
			glRotatef(45, 1.0, 0.0, 0.0);
			drawScaleCube(0.5, 0.15, 0.15); // a long snout
			glPushMatrix();
				glTranslatef(-0.5*0.5, 0.0, 0.0);
				//glRotatef(-45, 1.0, 0.0, 0.0);
				drawScaleCubeWithColour(0.07, 0.07, 0.07, 0.0, 0.0, 0.0);
			glPopMatrix();
			glPushMatrix();
				glTranslatef(0.0, 0.15, 0.0);
				drawEye(); //right eye
			glPopMatrix();
			glPushMatrix();
				glTranslatef(0.0, 0.0, -0.15);
				drawEye(); //left eye
			glPopMatrix();
			glPushMatrix();
				glTranslatef(0.0, -0.13, 0.0);
				drawSmile();
			glPopMatrix();
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.3*0.5, 0.3*0.5, 0.3*0.5); // left ear anchor point
			glRotatef(-10.0, 0.0, 0.0, 1.0); //un-tilt z
			glRotatef(10.0, 1.0, 0.0, 0.0); //the ears stick out a little
			drawEar(); // left ear
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.3*0.5, 0.3*0.5, -0.3*0.5); // right ear anchor point
			glRotatef(-10.0, 0.0, 0.0, 1.0);
			glRotatef(-10.0, 1.0, 0.0, 0.0);
			drawEar(); // right ear
		glPopMatrix();
	glPopMatrix();
}

void KHead::drawSmile()
{
	const float x = 0.05, y = 0.03, z = 0.15/2;
	glPushMatrix();
		glTranslatef( 0.0, 0.0, z/2 );
		drawScaleCubeWithColour(x, y, z, 1.0, 0.0, 0.0);
		glTranslatef( 0.0, 0.0, z/2 );
		glRotatef(-45, 1.0, 0.0, 0.0);
		glTranslatef( 0.0, 0.0, z/2 );
		drawScaleCubeWithColour(x, y, z, 1.0, 0.0, 0.0);
		glTranslatef( 0.0, 0.0, z/2 );
		glRotatef(-45, 1.0, 0.0, 0.0);
		glTranslatef( 0.0, 0.0, z/2 );
		drawScaleCubeWithColour(x, y, z, 1.0, 0.0, 0.0);
	glPopMatrix();
}

void KHead::drawEye()
{
		drawScaleCubeWithColour(0.065, 0.065, 0.065, 1.0, 1.0, 1.0);
		drawScaleCubeWithColour(0.07, 0.04, 0.04, 0.0, 0.0, 1.0);
}

void KHead::pushEarBack(float howFar)
{
	if( howFar >= 0 && howFar <= 1 )
	{
		ear_angle = howFar * 80;
	}
}

void KHead::drawEar()
{
	glPushMatrix();
		glRotatef(-1*ear_angle, 0.0, 0.0, 1.0);
		glTranslatef(0.0, 0.2*0.5, 0.0);
		drawScaleCube(0.1, 0.3, 0.1);
		glTranslatef(0.0, 0.3*0.5, 0.0);
		glRotatef(45.0, 1.0, 0.0, 0.0);
		drawScaleCube(0.0707,0.0707,0.0707); // the pointy bit
	glPopMatrix();
}

void KHead::toggle(bool smoothly)
{
	if( !smoothly )
	{
		switch( head_pos )
		{
			case 0:
				angle = 0.0;
				head_pos = 1;
				moving_from_left = true;
				break;
			case 1:
				if( !moving_from_left )
				{
					angle = min_swivel;
					head_pos = 0;
				}
				else
				{
					angle = max_swivel;
					head_pos = 2;
				}
				break;
			case 2:
				angle = 0.0;
				head_pos = 1;
				moving_from_left = false;
				break;
		}
	}
	else
	{
		if( !animating )
			animating = true;
	}
}
