#include "kleg.h"

KLeg::KLeg()
{
	animating = false;
	angle = min_leg_angle;
	delta = (max_leg_angle - min_leg_angle)/15;
	counter = 0;
}

void KLeg::draw(bool smoothly)
{
	if( smoothly && animating )
	{
		if( counter == 0 )
		{
			if(angle == min_leg_angle)
			{
				if(delta < 0) delta *= -1.0;
			}
			else
				if(delta > 0) delta *= -1.0;
			angle += delta;
			counter++;
		}
		else if( counter < 15 )
		{
			angle += delta;
			counter++;
		}
		else
		{
			if( delta < 0 )
				angle = min_leg_angle;
			else
				angle = max_leg_angle;
			counter = 0;
			animating = false;
		}
	}
	const float upper_x = 0.6;
	const float upper_y = 0.4;
	const float upper_z = 0.3;
	const float lower_x = 0.8;
	const float lower_y = 0.2;
	const float lower_z = 0.2;
	glPushMatrix();
		glRotatef(angle, 0.0, 0.0, 1.0);
		glTranslatef(-0.25*upper_x, 0.0, 0.0);
		drawScaleCube(upper_x, upper_y, upper_z); // upper leg
		glTranslatef(-0.5*upper_x, 0.0, 0.0);
		glRotatef(-2*angle-45, 0.0, 0.0, 1.0);
		glTranslatef(0.5*lower_x, 0.0, 0.0);
		drawScaleCube(lower_x, lower_y, lower_z); // lower leg
		glTranslatef(0.5*lower_x, 0.0, 0.0);
		glRotatef(45+angle, 0.0, 0.0, 1.0); // basically undoing previous rotations
		glTranslatef(0.5*lower_x-0.75*lower_x, 0.0, 0.0);
		drawScaleCube(0.75*lower_x, lower_y, lower_z); // paw
	glPopMatrix();
}

void KLeg::toggle(bool smoothly)
{
	if( !smoothly )
	{
		if( angle == min_leg_angle )
			angle = max_leg_angle;
		else
			angle = min_leg_angle;
	}
	else
	{
		if( !animating )
			animating = true;
	}
}

bool KLeg::isRaised()
{
	return (angle == min_leg_angle);
}
