#include "ktail.h"

KTail::KTail()
{
	tail_curl = min_curl;
	animate_tail = tail_at_top = false;
	delta = (max_curl - min_curl)/30;
}

void KTail::draw(bool smoothly)
{
	if( smoothly )
	{
		if( animate_tail )
		{
			if( tail_at_top && tail_curl > min_curl )
				tail_curl -= delta;
			else if( !tail_at_top && tail_curl < max_curl )
				tail_curl += delta;
			else
			{
				tail_at_top = !tail_at_top;
				animate_tail = false;
			}
		}
	}
	const float x = 0.3;
	const float y = 0.1;
	const float z = 0.1;
	glPushMatrix();
		glRotatef(-45.0, 0.0, 0.0, 1.0);
		for(int i = 0; i < 4; i++)
		{
			drawScaleCube(x, y, z);
			glTranslatef(x*0.5, 0, 0);
			glRotatef(tail_curl, 0.0, 0.0, 1.0);
			glTranslatef(x*0.5, 0, 0);
		}
		drawScaleCube(x, y, z);
	glPopMatrix();
}

void KTail::toggle(bool smoothly)
{
	if( !smoothly )
	{
		if(tail_at_top)
		{
			tail_curl = min_curl;
			tail_at_top = false;
		}
		else
		{
			tail_curl = max_curl;
			tail_at_top = true;
		}
	}
	else
	{
		if( !animate_tail )
			animate_tail = true;
	}
}

void KTail::setAngle(float angle)
{
	if( angle >= -1 && angle <= 1 )
	{
		float range = max_curl - min_curl;
		tail_curl = min_curl + angle*(range + min_curl)/2.0;
	}
}

bool KTail::atNormalPosition()
{
	return tail_curl == min_curl;
}
