#include "missle.h"

Missle::Missle(void)
:Object()
{
	collisionInfo.boundingRadius = 25;
	isFlying = true;
	reset();
	damage = 25;
	hitPoints = 1;
}

Missle::~Missle(void)
{
	gluDeleteQuadric(quad);
}

void Missle::init(CollisionManager * collisionManager, Fire* fire)
{
	this->pCollisionManager = collisionManager;
	this->fire = fire;
	quad = gluNewQuadric();
	gluQuadricNormals(quad, GLU_SMOOTH);
}

void Missle::launch(Vertex * startPosition,Arrow * velocity,double startSpeed,Object * parent)
{
	position = *startPosition;
	collisionInfo.checkAgainstMe = true;
	unitVelocity = *velocity;
	speed = startSpeed;
	this->parent = parent;
	//isLaunched = true;
	state = MISSLE_LAUNCHED;
	timer = MyTime(15.0);
}
void Missle::reset()
{
	collisionInfo.checkAgainstMe = false;
	state = MISSLE_RESET;
}
void Missle::startExploding()
{
	collisionInfo.checkAgainstMe = false;
	state = MISSLE_EXPLODING;
	explosion.reset();	
}
void Missle::drawBody()
{
	if(state == MISSLE_LAUNCHED)
	{
		double rotAngle;
		Arrow axis = unitVelocity.cross(zAxis);	
		rotAngle =unitVelocity.angleBetween(&zAxis,&axis);
	
		glPushMatrix();
		glTranslatef(position.getX(),position.getY(),position.getZ());
		glRotatef(rotAngle * -1,axis.getX(),axis.getY(),axis.getZ());
	
		glColor3f(1,1,1);
		glPushMatrix();
			glTranslatef(0,0,40);
			gluDisk(quad, 0, 10, 10, 10);
		glPopMatrix();
		glTranslatef(0,0,20);
		gluCylinder(quad, 10, 1, 60, 10, 10);
		glTranslatef(0,0,-20);
		gluDisk(quad, 0, 10, 10, 10);
		glRotatef(-90, 1, 0, 0);
		glScalef(15,40,15);
		fire->draw();
		glPopMatrix();
	}
	else if(state == MISSLE_EXPLODING)
	{
		glPushMatrix();
		glTranslatef(position.getX(),position.getY(),position.getZ());
		glScalef(4.0,4.0,4.0);
		bool alive = explosion.draw();
		glPopMatrix();
		if(!alive)
		{
			reset();
			//printf("done exploding\n");
		}
	}
}
void Missle::update()
{
	//printf("%f\n", speed);
	if(hitPoints <= 0)
	{
		hitPoints = 1;
		startExploding();
		return;
	}
	if(state == MISSLE_LAUNCHED)
	{
		if(timer.check())
		{
			startExploding();
			return;
		}
		unitVelocity.setLength(1);
		Arrow increment = unitVelocity.mul(speed);
		pCollisionManager->RunCollisionsCheck(this);
		if(collisionInfo.collisionFound)
		{
			//printf("collided\n");
			startExploding();
			return;
		}
		else
		{
			position = position.add(&increment);
		}
	}
	else if(state == MISSLE_EXPLODING)
	{
	}	
}


