#ifndef OBJECT_CPP
#define OBJECT_CPP

#include "common.h"
#include "geometry.cpp"
extern bool drawBoundingBoxes;
struct CollisionInfo
{
	double boundingRadius;
	bool collisionFound;
	Vertex lastGoodPoint;
	Arrow slideDirection;
	bool checkAgainstMe;
};
class Object
{
public:
	bool isFlying;
	CollisionInfo collisionInfo;
	Vertex unitVelocity;
	Vertex position;
	Vertex startPos;
	Arrow yAxis;
	Arrow xAxis;
	Arrow zAxis;	
	int hitPoints;
	double speed;
	Object * parent;
	double maxSpeed;
	bool isEnemy;
	bool isHero;
	int damage;
	Object(void)
	{
		unitVelocity.set(0,0,1);
		position.set(0,0,0);
		startPos.set(0,0,0);
		yAxis.set(0,1,0);		
		xAxis.set(1,0,0);		
		zAxis.set(0,0,1);	
		collisionInfo.boundingRadius = 30;
		collisionInfo.collisionFound = false;
		collisionInfo.checkAgainstMe = true;
		parent = 0;
		this->hitPoints = 100; //default
		isEnemy = false;
		isHero = false;
		damage = 0;
	}
	~Object(void)
	{

	}

	void draw()
	{
		update();
		drawBody();	
		if(drawBoundingBoxes)
			drawBoundingBox();
	}
	
	void drawBoundingBox()
	{
		glPushMatrix();
		glColor3f(0,1,0);
		glTranslatef(position.getX(),position.getY(),position.getZ());
		glutWireSphere(collisionInfo.boundingRadius,12,12);		
		glPopMatrix();
	}
	
	virtual void drawBody() =0;
	virtual void update() {};

	virtual void hit(Object *source)
	{
		Arrow adjustedHitDir = source->unitVelocity.mul(0.05);
		unitVelocity = this->unitVelocity.add(&adjustedHitDir);
		unitVelocity.setLength(1);
		this->hitPoints -= source->damage;
	}

};
#endif



