#ifndef P3_CPP
#define P3_CPP

#include "common.h"
#include "terrain.cpp"
#include "geometry.cpp"
#include "face.cpp"
#include "corner.cpp"
#include "texture.h"
#include "cockpit.h"
#include "eviltank.cpp"
#include "hero.cpp"
#include "collisionmanager.h"
#include "evilchopper.cpp"
#include "clouds.h"
#include "tree.h"
#include "tower.h"
#include "radar.cpp"
#include "missilemanager.h"

#define NUM_ENEMIES 6

TEXTURE metalTex;
TEXTURE panelTex;
TEXTURE camoTex;
TEXTURE wheelTex;
TEXTURE wallTex;
TEXTURE dialTex;
TEXTURE waterTex;
TEXTURE treeTex;

double gCurrentMatrix[16];

Terrain gTerrain;
Clouds clouds;
EvilChopper gEvilChopper[NUM_ENEMIES];
Tower tower;
Hero gHero;
EvilTank gEvilTank[NUM_ENEMIES];
int numTrees = 20;
Tree gTree[20];

CollisionManager gCollisionManager(&gTerrain);
Radar radar(&gCollisionManager);
MissileManager mmanager(&gCollisionManager);

int gButtonState = -1;	//saves mouse state for mouse mode
int gButtonType = -1;
int gMouseX = -1;
int gMouseY = -1;
bool gEnableHeadlight = true;	//enable head light toggle
bool mouseMotionEnabled = true;
double waterShift =0;
bool drawBoundingBoxes = false;
bool rolling = false; int roll_start_x = 0;
/*
* Template's keyboard callback
*/
void ResetView()	//resets the view to current camera setup.I dont think i use this anywhere
{
	glLoadIdentity();
	gluLookAt(gHero.position.getX(),gHero.position.getY(),gHero.position.getZ(),
				gHero.lookAtPoint.getX(),gHero.lookAtPoint.getY(),gHero.lookAtPoint.getZ(),
				gHero.upAxis.getX(),gHero.upAxis.getY(),gHero.upAxis.getZ());
	 
	glGetDoublev(GL_MODELVIEW_MATRIX,gCurrentMatrix);
}

void KeyboardCallback(unsigned char c, int x, int y) {
  switch (c) 
  {
	case '~':
		exit(0);
		break;
// 	case 'u':
// 		gTerrain.regenerate();
// 		break;
// 	case '+':
// 		gTerrain.nHorizontalPoints = min(gTerrain.nHorizontalPoints + 25,MAX_HORZ_POINTS);
// 		gTerrain.nVerticalPoints = gTerrain.nHorizontalPoints;
// 		gTerrain.regenerate();
// 		break;
// 	case '-':
// 		gTerrain.nHorizontalPoints = max(gTerrain.nHorizontalPoints - 25,25);
// 		gTerrain.nVerticalPoints = gTerrain.nHorizontalPoints;
// 		gTerrain.regenerate();
// 		break;	
// 	case '/':
// 		gTerrain.nBumpFraction = max(gTerrain.nBumpFraction - 0.2,0); 
// 		gTerrain.regenerate();
// 		break;
// 	case '*':
// 		gTerrain.nBumpFraction = min(gTerrain.nBumpFraction + 0.2,8); 
// 		gTerrain.regenerate();
// 		break;
		
// 	case 'f':
// 		gTerrain.bDoFlatShading = !gTerrain.bDoFlatShading; 
// 		break;
// 	case 'v':
// 		gTerrain.bPerFaceNormals= !gTerrain.bPerFaceNormals; 
// 		break;
// 	case 'n':
// 		gTerrain.bDrawNormals = !gTerrain.bDrawNormals; 
// 		break;
// 	case 'r':
// 		break;
	case 'b':
		drawBoundingBoxes = !drawBoundingBoxes;
		break;
	
	case'h':
		{
		gEnableHeadlight = !gEnableHeadlight;
		if(gEnableHeadlight)
			glEnable(GL_LIGHT1);
		else
			glDisable(GL_LIGHT1);		
		}
		break;	 
   }
 
  	switch(c)
	{
		case 'w':
			gHero.changeSpeed(true);
			
			break;
		case 's':
			gHero.changeSpeed(false);
			
			break;
		case 'd':
			gHero.changePan(false,0.05);
			break;
		case 'a':
			gHero.changePan(true,0.05);
			break;
		case ' ':
			{
				mouseMotionEnabled = !mouseMotionEnabled;
				break;
			}
		case 'x':
			{
				gHero.changeSpeed(false);
				break;
			}
		case 'q':
			gHero.changeRoll(false,0.05);
			break;
		case 'e':
			gHero.changeRoll(true,0.05);
	}
  glutPostRedisplay();
}

void DisplayCallback()
{
  // clear the color buffer
  glLoadIdentity();
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 
  // set up camera
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluPerspective( 45, 1.0, 3, 100 );
  
  glMatrixMode( GL_MODELVIEW );  
  
  glPushMatrix();
  gHero.draw();
  
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluPerspective( 45, 1.0, 20, RIGHT*2 );
  glMatrixMode( GL_MODELVIEW );    
  
  gluLookAt(gHero.position.getX(),gHero.position.getY(),gHero.position.getZ(),
				gHero.lookAtPoint.getX(),gHero.lookAtPoint.getY(),gHero.lookAtPoint.getZ(),
				gHero.upAxis.getX(),gHero.upAxis.getY(),gHero.upAxis.getZ());
	
   clouds.draw();

	
	
  //display global light
  GLfloat ambient[] = {0.6, 0.6, 0.6, 0.6};
  GLfloat diffuse[] = {0.7, 0.7, 0.7, 0.7};
  GLfloat specular[] = {0.7, 0.7, 0.7, 0.7};
  GLfloat position0[] = {(RIGHT - LEFT)/2, 30000, (FAR - NEAR)/2, 0.0};
  glLightfv( GL_LIGHT0, GL_POSITION, position0 );
  glLightfv( GL_LIGHT0, GL_AMBIENT, ambient );
  glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuse );
  glLightfv( GL_LIGHT0, GL_SPECULAR, specular );
  glEnable( GL_LIGHT0);
  
  //draw terrain
   gTerrain.draw();
   
   
   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D,waterTex.TexID);		
   
   glMatrixMode( GL_TEXTURE );    //texture
   glPushMatrix();
   glTranslatef(waterShift,0,0);
   glScalef(4.0, 4.0, 4.0);
   
   glMatrixMode( GL_MODELVIEW );    
   glPushMatrix(); //modelview
   glColor3f(0.9,0.9,1);
   glTranslatef(RIGHT / 2.0, 0, FAR / 2.0);	    
   DrawTexturedBox(RIGHT * 2,3000,(FAR - NEAR)*2);  
   glPopMatrix();
   
   
   glMatrixMode( GL_TEXTURE );    
   glPopMatrix();
   glMatrixMode( GL_MODELVIEW);    
   
   glDisable(GL_TEXTURE_2D);
   
   waterShift -= 0.0002;
   
   //wall.draw();
   tower.draw();
   
   
   glColor3f(0.8,0.8,0.8);
   //PUT MORE GLOBAL THINGS HERE
   for(int i = 0; i < NUM_ENEMIES; i++)
   {
   	gEvilTank[i].draw();
   	gEvilChopper[i].draw();
   }
   
   for(int i=0;i<numTrees;i++)
		gTree[i].draw();
   
   gHero.drawShootingVector();
  
   //
   mmanager.drawMissiles();
   
   glPopMatrix();  
   
   radar.draw();
   
  //done drawing
  glutSwapBuffers();

}

void IdleCallback()
{
	double width = glutGet(GLUT_WINDOW_WIDTH);
	double height = glutGet(GLUT_WINDOW_HEIGHT);	
	double xCenter = width /2;
	double yCenter = height /2;			
	double xDiff = gMouseX - xCenter;
 	double yDiff = gMouseY - yCenter;

	gHero.changeAim(gMouseX,height -gMouseY);
	double multiplier = 0.10;
	if(mouseMotionEnabled && !rolling)
	{
		if(absD(xDiff) > width / 12)
		{
			if(xDiff < 0)
				gHero.changePan(true,xDiff / width * - multiplier);
			else
				gHero.changePan(false,xDiff / width * multiplier);	
		}
		if(absD(yDiff) > height/ 12)
		{
						
			if(yDiff < 0)
				gHero.changePitch(true,yDiff / width * -multiplier);
			else
				gHero.changePitch(false,yDiff / width * multiplier);	
		}
	}
	
	if(rolling)
	{
		int diff_x = gMouseX - roll_start_x;
		if( diff_x < 0 )
		{
			gHero.changeRoll(false, (diff_x/2000.0));
		}
		else
		{
			gHero.changeRoll(true, (diff_x/-2000.0));
		}
	}
		
	glutPostRedisplay();
}

void MotionCallback(int x,int y)
{
	gMouseX = x;
	gMouseY = y;	
}

void MouseCallback(int button, int state,int x, int y)
{
	if(state == GLUT_DOWN && button == GLUT_LEFT_BUTTON)
		gHero.fireMissle();
	else if(button == GLUT_RIGHT_BUTTON)
	{
		if(state == GLUT_DOWN)
		{
			rolling = true;
			roll_start_x = x;
		}
		else
		{
			rolling = false;
		}
	}
}
void setupObjects()
{
	Vertex spawnPoints[NUM_ENEMIES];
	spawnPoints[0].set(LEFT+40000,30000,NEAR-40000);
	spawnPoints[1].set(LEFT+40000,30000,FAR+40000);
	spawnPoints[2].set(RIGHT-40000,30000,NEAR-40000);
	spawnPoints[3].set(RIGHT-40000,30000,FAR+40000);
	spawnPoints[4].set((RIGHT-LEFT)/2, 30000, NEAR-40000);
	spawnPoints[5].set((RIGHT-LEFT)/2, 30000, FAR+40000);
	//spawnPoints[6].set((RIGHT-LEFT)/4, 30000, NEAR-40000);
	//spawnPoints[7].set((RIGHT-LEFT)*3/4, 30000, FAR+40000);
	
	Vertex heroSpawn(LEFT+70000, 30000, NEAR-70000);
	Vertex middleOfTerrain((RIGHT - LEFT)/2,12000,(FAR - NEAR)/2);
	clouds.init();
	tower.init(&middleOfTerrain,&gTerrain);
	
	gHero.init(&gCollisionManager,&heroSpawn, &mmanager);
	
	for(int i = 0; i < NUM_ENEMIES; i++)
	{
		gEvilTank[i].init(&gTerrain,&gHero,&gCollisionManager,&spawnPoints[i],&mmanager);
		gEvilChopper[i].init(&gHero,&gCollisionManager,&spawnPoints[i],&gTerrain,&mmanager);
	}
	
	Vertex treeStart;
	for(int i = 0;i < numTrees;i++)
	{
		treeStart.set(myRandom(LEFT + 30000,RIGHT - 30000),0,myRandom(FAR + 30000,NEAR - 30000));
		gTree[i].init(&treeStart,&gTerrain);
		gCollisionManager.collisionList.push_back(&gTree[i]);
	}
	
	//set up collision stuff
	gCollisionManager.collisionList.push_back(&gHero);
	for(int i = 0; i < NUM_ENEMIES; i++)
	{
		gCollisionManager.collisionList.push_back(&gEvilTank[i]);
		gCollisionManager.collisionList.push_back(&gEvilChopper[i]);
	}
	gCollisionManager.collisionList.push_back(&tower);
}
int main(int argc, char **argv)
{
	// create window and rendering context
	glutInit( &argc, argv );
	glutInitDisplayMode( GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE );
	glutInitWindowSize( 500, 500 );
	glutCreateWindow( "p3: game" );
	//setup GLU
	
	// register display callback
	glutDisplayFunc( DisplayCallback );
	glutKeyboardFunc( KeyboardCallback );
	glutPassiveMotionFunc(MotionCallback); glutMotionFunc(MotionCallback);
	glutMouseFunc(MouseCallback);
	glutIdleFunc(IdleCallback);
	glViewport( 0, 0, 500, 500 );
	glEnable( GL_DEPTH_TEST );
	
	// lighting stuff
	glEnable( GL_COLOR_MATERIAL );
	glEnable( GL_LIGHTING );
	glEnable( GL_BLEND );
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45, 1.0, 3, RIGHT*2 );
	
	glEnable(GL_NORMALIZE);
	glClearColor(0.5,0.8,1.0,0.0);
	//glClearColor(0.7,0,0,0.3);
	GLfloat diffuse[] = {1, 1, 1, 1};
	GLfloat specular[] = {1, 1, 1, 1};
	GLfloat position[] = {0, 0, -5, 1.0};
	GLfloat direction[] = {0, 0, -1, 0.0};
	GLfloat exponent = 128;
	GLfloat cutoff = 50;
	glLightfv( GL_LIGHT1, GL_POSITION, position );
	glLightfv( GL_LIGHT1, GL_DIFFUSE, diffuse );
	glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION,direction);
	glLightfv( GL_LIGHT1, GL_SPECULAR, specular);
	glLightfv(GL_LIGHT1,GL_SPOT_CUTOFF,&cutoff);
	glLightfv(GL_LIGHT1,GL_SPOT_EXPONENT,&exponent);
//	glEnable( GL_LIGHT1);
	
	
	
	glutSetCursor(GLUT_CURSOR_CROSSHAIR);
	glMatrixMode( GL_MODELVIEW );
	ResetView();
	glGetDoublev(GL_MODELVIEW_MATRIX,gCurrentMatrix);
	
	sprintf(gTerrain.texture[0].TexName,"Textures/terrain.tga");
	for(int i =0;i< 1;i++)
	{
		if(!gTerrain.texture[i].LoadTGA())
		{
			printf("\nCouldn't load texture(%s). Returning",gTerrain.texture[i].error);
			return 0;
		}
	}
	sprintf(metalTex.TexName,"Textures/panel.tga");
	if(!metalTex.LoadTGA())
	{
		printf("\nCouldn't load texture(%s). Returning",metalTex.error);
		return 0;
	}
	sprintf(camoTex.TexName,"Textures/army_camo.tga");
	if(!camoTex.LoadTGA())
	{
		printf("\nCouldn't load texture(%s). Returning",camoTex.error);
		return 0;
	}
	sprintf(wheelTex.TexName,"Textures/wheel.tga");
	if(!wheelTex.LoadTGA())
	{
		printf("\nCouldn't load texture(%s). Returning",wheelTex.error);
		return 0;
	}
	sprintf(wallTex.TexName, "Textures/stonewallDiffuse.tga");
	if(!wallTex.LoadTGA())
	{
		printf("\nCouldn't load texture(%s). Returning",wallTex.error);
		return 0;
	}
	sprintf(dialTex.TexName, "Textures/dial.tga");
	if(!dialTex.LoadTGA())
	{
		printf("\nCouldn't load texture(%s). Returning",dialTex.error);
		return 0;
	}
	sprintf(waterTex.TexName, "Textures/water.tga");
	if(!waterTex.LoadTGA())
	{
		printf("\nCouldn't load texture(%s). Returning",waterTex.error);
		return 0;
	}
	sprintf(treeTex.TexName, "Textures/bark1.tga");
	if(!treeTex.LoadTGA())
	{
		printf("\nCouldn't load texture(%s). Returning",treeTex.error);
		return 0;
	}
 //	glEnable(GL_FOG);
 //	GLfloat fogcolor[4] = { 0.5,0.8,1.0,0.0 }; //same as clear color
// 	glFogi(GL_FOG_MODE, GL_LINEAR);
 	//glFogf(GL_FOG_DENSITY, 0.35);
//	glFogf(GL_FOG_START, RIGHT/2);
//	glFogf(GL_FOG_END, RIGHT);
	
	glPointSize(2.0);
	glLineWidth(2.0);
	setupObjects();
	glutMainLoop();
	return 0;       // never reached
}
#endif




