#include "perlin2d.h"

/*
This is adapted from the tutorials at
http://freespace.virgin.net/hugo.elias/models/m_perlin.htm and
http://www.gamedev.net/reference/articles/article2085.asp and
mostly http://www.robo-murito.net/code/perlin-noise-math-faq.html
*/

PerlinNoise2D::PerlinNoise2D(int size, int freq)
{
	rnd_x = random();
	rnd_y = random();
	this->size = size;
	this->freq = freq;
	sf = size / freq;
	noise = new float*[size];
	for(int i = 0; i < size; i++)
		noise[i] = new float[size];
	genNoise();
}

PerlinNoise2D::~PerlinNoise2D()
{
	for(int i = 0; i < size; i++)
		delete [] noise[i];
	delete [] noise;
}

float*::PerlinNoise2D::operator[](const int inx) const
{
	return noise[inx];
}

float PerlinNoise2D::prng(int x, int y, int rnd)
{
	int n = x + y * 57 + rnd * 131;
    n = (n<<13) ^ n;
    return (1.0f - ( (n * (n * n * 15731 + 789221) +
            1376312589)&0x7fffffff)* 0.000000000931322574615478515625f);
}

float pair_dot(const std::pair<float,float>& v1, const std::pair<float,float>& v2)
{
	return v1.first * v2.first + v1.second*v2.first;
}

void PerlinNoise2D::genNoise()
{
	//generate gradients at every sf*sf grid point
	std::pair<float, float> gradients[freq][freq];
	for(int x = 0; x < freq; x++)
	for(int y = 0; y < freq; y++)
	{
		float x_grad = prng(x, y, rnd_x);
		float y_grad = prng(x, y, rnd_y);
		float len = sqrt(x_grad*x_grad + y_grad*y_grad);
		x_grad /= len;
		y_grad /= len;
		gradients[x][y] = std::make_pair(x_grad, y_grad);
	}
	
	for(int x_sq = 0; x_sq < freq; x_sq++)
	for(int y_sq = 0; y_sq < freq; y_sq++)
	{
		std::pair<float, float> grad0, grad1, grad2, grad3;
		grad0 = gradients[x_sq][y_sq];
		int x1, y1;
		if( x_sq == freq - 1 )
			x1 = 0;
		else
			x1 = x_sq + 1;
		if(y_sq == freq - 1)
			y1 = 0;
		else
			y1 = y_sq + 1;
		
		grad1 = gradients[x1][y_sq];
		grad2 = gradients[x_sq][y1];
		grad3 = gradients[x1][y1];
		
		for(int x = 0; x < sf; x++)
		for(int y = 0; y < sf; y++)
		{
			float f_x = x/(float)sf;
			float f_y = y/(float)sf;
			float s = pair_dot(grad0, std::make_pair(f_x,f_y));
			float t = pair_dot(grad1, std::make_pair(f_x-1,f_y));
			float u = pair_dot(grad2, std::make_pair(f_x,f_y-1));
			float v = pair_dot(grad3, std::make_pair(f_x-1,f_y-1));
			
			float Sx = 3.0*(f_x*f_x) - 2*(f_x*f_x*f_x);
			float a = s + Sx*(t - s);
			float b = u + Sx*(v - u);
			
			float Sy = 3.0*(f_y*f_y) - 2.0*(f_y*f_y*f_y);
			float z = a + Sy*(b - a);
			
			noise[x_sq*sf + x][y_sq*sf + y] = z;
		}
	}
}
