#include "tower.h"

Tower::Tower()
{
	size = RIGHT * 0.005;
	collisionInfo.boundingRadius = 5000;
	time = MyTime(1 / 15.0);
	noise = new PerlinNoise2D(FLAGSIZE,FLAGSIZE/8);
	Corner b1(-10, 0, 10);
	Corner b2(10, 0, 10);
	Corner b3(8, 45, 8);
	Corner b4(-8, 45, 8);
	Corner b5(-10, 0, -10);
	Corner b6(10, 0, -10);
	Corner b7(8, 45, -8);
	Corner b8(-8, 45, -8);
	base.set(&b1, &b2, &b3, &b4, &b5, &b6, &b7, &b8);
	
	Corner s1(0,0,0);
	Corner s2(1,0,0);
	Corner s3(1,10,0);
	Corner s4(0,10,0);
	Corner s5(0,0,-1);
	Corner s6(1,0,-1);
	Corner s7(1,10,-1);
	Corner s8(0,10,-1);
	support.set(&s1, &s2, &s3, &s4, &s5, &s6, &s7, &s8);
	
	Corner r1(-8,0,8);
	Corner r2(8,0,8);
	Corner r3(0,5,0);
	Corner r4(0,5,0);
	Corner r5(-8,0,-8);
	Corner r6(8,0,-8);
	Corner r7(0,5,0);
	Corner r8(0,5,0);
	roof.set(&r1, &r2, &r3, &r4, &r5, &r6, &r7, &r8);
}

void Tower::init(Vertex * startPos,Terrain * terrain)
{
	well.init();
	double height = 10000;
	this->terrain = terrain;
	double y = terrain->heightFunction(startPos->getX(),startPos->getZ())- 1000;
	this->startPos = *startPos;
	position = *startPos;
	
	this->startPos.setY(y + height/2);
	position.setY(y + height/2);
	
	quad = gluNewQuadric();
	gluQuadricNormals(quad, GLU_SMOOTH);
	sprintf(concrete.TexName, "Textures/cobblestonesDiffuse.tga");
	if(!concrete.LoadTGA())
	{
		printf("\nCouldn't load texture (%s).\n",concrete.error);
	}
	sprintf(xkang.TexName, "Textures/xkangaroo.tga");
	if(!xkang.LoadTGA())
	{
		printf("\nCouldn't load texture (%s).\n",xkang.error);
	}
	step = 0;
	Face* face = &faces[0];
	Corner* c1, *c2, *c3, *c4;
	
	for(int y = 0; y < FLAGSIZE; y++)
	for(int x = 0; x < FLAGSIZE; x++)
	{
		nodes[x][y].setX(((double)x / FLAGSIZE)*14.0);
		nodes[x][y].setY(((double)y / FLAGSIZE)*7.0);
	}
	
	for(int y = 0; y < FLAGSIZE-1; y++)
	for(int x = 0; x < FLAGSIZE-1; x++)
	{
		///this part is scary
		c1 = &nodes[y][x];
		c2 = &nodes[y][x+1];
		c3 = &nodes[y+1][x+1];
		c4 = &nodes[y+1][x];
		
		* face = Face();
		face->addVertex(c1);
		face->addVertex(c2);
		face->addVertex(c4);
		c1->addFace(face);
		c2->addFace(face);
		c4->addFace(face);
		
		face++;
		* face = Face();
		face->addVertex(c2);
		face->addVertex(c3);
		face->addVertex(c4);
		c2->addFace(face);
		c3->addFace(face);
		c4->addFace(face);
		face++;
	}
	updateFlag();
}

Tower::~Tower()
{
	gluDeleteQuadric(quad);
	delete noise;
}

void Tower::drawBody()
{
	double scale = size / 8.0;
	glPushMatrix();
	glTranslatef(position.getX(), position.getY() - 5000,position.getZ());
	glScalef(scale, scale, scale);
	drawTower();
	glPopMatrix();
	glDisable(GL_TEXTURE_2D);	
}

void Tower::drawTower()
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, concrete.TexID);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	
	
	glColor3f(1,1,1);
	base.draw();
	glPushMatrix();
		//glScalef(0.5, 1, 0.5);
		glTranslatef(-8,45,8);
		glPushMatrix();
			support.draw(); //support poles
			glTranslatef(15,0,0);
			support.draw();
			glTranslatef(0,0,-15);
			support.draw();
			glTranslatef(-15,0,0);
			support.draw();
		glPopMatrix();
		//draw top
		glTranslatef(8,10,-8);
		roof.draw();
// 		fire.disableLight();
		glTranslatef(0,4.8,0);
		glDisable(GL_TEXTURE_2D);
		glColor3f(0.5, 0.5, 0.5);
		glPushMatrix();
			glRotatef(-90, 1, 0, 0);
			gluCylinder(quad, 0.3, 0.3, 20, 10, 10);
		glPopMatrix();
		glTranslatef(0,20,0);
		glutSolidSphere(0.8, 10, 10);
		glTranslatef(0.4, -8, 0);
		drawFlag();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0,45,0);
		glScalef(2, 2, 2);
		glPushMatrix();
			glTranslatef(0, 2, 0);
			roo.drawIt();
			roo.toggleHop();
		glPopMatrix();
		well.draw();
		fire.draw();
	glPopMatrix();
}

void Tower::update()
{
	fire.update();
	testAndUpdate();
}

void Tower::testAndUpdate()
{
	if(time.check())
	{
		updateFlag();
	}
}

void Tower::updateFlag()
{
	for(int x = 0; x < FLAGSIZE; x++)
	{
		for(int y = 0; y < FLAGSIZE; y++)
		{
			nodes[x][y].setZ( (double)((*noise)[step][y]) );
		}
		step = (step + 1) % FLAGSIZE;
	}
	step = (step + 1) % FLAGSIZE;
	
	//calculate new normals for faces
	//for(int y = 0; y < FLAGSIZE-1; y++)
	for(int x = 0; x < (FLAGSIZE-1)*(2*(FLAGSIZE-1)); x++)
	{
		faces[x].calculateNormal();
	}
	
	//average new normals at vertices
	for(int x = 0; x < FLAGSIZE; x++)
	for(int y = 0; y < FLAGSIZE; y++)
	{
		nodes[x][y].calculateNormal();
	}
}

void Tower::drawFlag()
{
	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, xkang.TexID);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glEnable(GL_LIGHTING);
	
	for(int y = 1; y < FLAGSIZE; y++)
	{
		glColor3f(1,1,1);
		glBegin(GL_TRIANGLE_STRIP);
		for(int x = 0; x < FLAGSIZE-1; x++)
		{
			glTexCoord2f(float(x) / (FLAGSIZE-1), float(y) / (FLAGSIZE-1));
			glNormal3dv(nodes[y][x].getNormal()->getCoords());
			glVertex3dv(nodes[y][x].getCoords());
			
			glTexCoord2f(float(x) / (FLAGSIZE-1), float(y-1) / (FLAGSIZE-1));
			glNormal3dv(nodes[y-1][x].getNormal()->getCoords());
			glVertex3dv(nodes[y-1][x].getCoords());
			
			glTexCoord2f(float(x+1) / (FLAGSIZE-1), float(y) / (FLAGSIZE-1));
			glNormal3dv(nodes[y][x+1].getNormal()->getCoords());
			glVertex3dv(nodes[y][x+1].getCoords());
			
			glTexCoord2f(float(x+1) / (FLAGSIZE-1), float(y-1) / (FLAGSIZE-1));
			glNormal3dv(nodes[y-1][x+1].getNormal()->getCoords());
			glVertex3dv(nodes[y-1][x+1].getCoords());
		}
		glEnd();
		
// 		glColor3f(1,0,0);
// 		glBegin(GL_LINES);
// 		for(int x = 0; x < FLAGSIZE-1; x++)
// 		{
// 			DrawArrow(nodes[y][x].getNormal()->getCoords(), nodes[y][x].getCoords(), 1, 1, 0, 0);
// 		}
// 		glEnd();
	}
	
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	//glEnable(GL_LIGHTING);
}
